"""Mock token models for claudable_helper.

This module provides mock implementations of the token models
that were originally imported from app.models.tokens.
"""
import uuid
from datetime import datetime, timedelta
from enum import Enum
from typing import Any, Dict, Optional


class TokenType(str, Enum):
    """Token type enumeration."""
    API_KEY = "api_key"
    ACCESS_TOKEN = "access_token"
    REFRESH_TOKEN = "refresh_token"
    SERVICE_TOKEN = "service_token"


class TokenStatus(str, Enum):
    """Token status enumeration."""
    ACTIVE = "active"
    EXPIRED = "expired"
    REVOKED = "revoked"
    INVALID = "invalid"


class ServiceToken:
    """Mock service token model for managing API tokens."""
    
    def __init__(
        self,
        service_name: str,
        token_value: str,
        token_type: TokenType = TokenType.API_KEY,
        token_id: Optional[str] = None,
        description: Optional[str] = None,
        expires_at: Optional[datetime] = None,
        metadata: Optional[Dict[str, Any]] = None,
        created_at: Optional[datetime] = None,
        updated_at: Optional[datetime] = None,
    ):
        self.id = token_id or str(uuid.uuid4())
        self.service_name = service_name
        self.token_value = token_value
        self.token_type = token_type
        self.description = description
        self.expires_at = expires_at
        self.metadata = metadata or {}
        self.created_at = created_at or datetime.utcnow()
        self.updated_at = updated_at or datetime.utcnow()
    
    @property
    def status(self) -> TokenStatus:
        """Get current token status."""
        if self.expires_at and self.expires_at < datetime.utcnow():
            return TokenStatus.EXPIRED
        
        # In a real implementation, you might check if the token is valid
        # by making an API call or checking a database
        return TokenStatus.ACTIVE
    
    @property
    def is_valid(self) -> bool:
        """Check if token is valid and not expired."""
        return self.status == TokenStatus.ACTIVE
    
    @property
    def is_expired(self) -> bool:
        """Check if token is expired."""
        return self.status == TokenStatus.EXPIRED
    
    @property
    def masked_token(self) -> str:
        """Get masked version of the token for display."""
        if len(self.token_value) <= 8:
            return "*" * len(self.token_value)
        
        return self.token_value[:4] + "*" * (len(self.token_value) - 8) + self.token_value[-4:]
    
    def set_expiry(self, days: int) -> None:
        """Set token expiry to specified number of days from now."""
        self.expires_at = datetime.utcnow() + timedelta(days=days)
        self.updated_at = datetime.utcnow()
    
    def revoke(self) -> None:
        """Revoke this token."""
        self.metadata["revoked"] = True
        self.metadata["revoked_at"] = datetime.utcnow().isoformat()
        self.updated_at = datetime.utcnow()
    
    @property
    def is_revoked(self) -> bool:
        """Check if token is revoked."""
        return self.metadata.get("revoked", False)
    
    def to_dict(self, include_token: bool = False) -> Dict[str, Any]:
        """Convert token to dictionary.
        
        Args:
            include_token: Whether to include the actual token value (dangerous!)
        """
        data = {
            "id": self.id,
            "service_name": self.service_name,
            "token_type": self.token_type.value,
            "description": self.description,
            "status": self.status.value,
            "expires_at": self.expires_at.isoformat() if self.expires_at else None,
            "metadata": self.metadata,
            "created_at": self.created_at.isoformat() if self.created_at else None,
            "updated_at": self.updated_at.isoformat() if self.updated_at else None,
        }
        
        if include_token:
            data["token_value"] = self.token_value
        else:
            data["masked_token"] = self.masked_token
        
        return data
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ServiceToken":
        """Create token from dictionary."""
        return cls(
            token_id=data.get("id"),
            service_name=data.get("service_name", ""),
            token_value=data.get("token_value", ""),
            token_type=TokenType(data.get("token_type", "api_key")),
            description=data.get("description"),
            expires_at=datetime.fromisoformat(data["expires_at"]) if data.get("expires_at") else None,
            metadata=data.get("metadata", {}),
            created_at=datetime.fromisoformat(data["created_at"]) if data.get("created_at") else None,
            updated_at=datetime.fromisoformat(data["updated_at"]) if data.get("updated_at") else None,
        )
    
    def __str__(self) -> str:
        """String representation of the token."""
        return f"ServiceToken(service={self.service_name}, type={self.token_type.value})"
    
    def __repr__(self) -> str:
        """Detailed string representation."""
        return (
            f"ServiceToken(id={self.id}, service={self.service_name}, "
            f"type={self.token_type.value}, status={self.status.value})"
        )


"""
GitHub API service for repository management
"""
import httpx
import json
from typing import Dict, Any, Optional
from urllib.parse import quote
import logging

logger = logging.getLogger(__name__)


class GitHubAPIError(Exception):
    """Custom exception for GitHub API errors"""
    def __init__(self, message: str, status_code: int = None):
        self.message = message
        self.status_code = status_code
        super().__init__(self.message)


class GitHubService:
    """GitHub API service for repository operations"""
    
    BASE_URL = "https://api.github.com"
    
    def __init__(self, token: str):
        self.token = token
        self.headers = {
            "Authorization": f"token {token}",
            "Accept": "application/vnd.github.v3+json",
            "User-Agent": "Clovable/1.0"
        }
    
    async def check_token_validity(self) -> Dict[str, Any]:
        """Check if the GitHub token is valid and get user info"""
        async with httpx.AsyncClient() as client:
            try:
                response = await client.get(
                    f"{self.BASE_URL}/user",
                    headers=self.headers
                )
                
                if response.status_code == 200:
                    user_data = response.json()
                    return {
                        "valid": True,
                        "username": user_data.get("login"),
                        "name": user_data.get("name"),
                        "email": user_data.get("email"),
                        "avatar_url": user_data.get("avatar_url")
                    }
                elif response.status_code == 401:
                    return {"valid": False, "error": "Invalid or expired token"}
                else:
                    return {"valid": False, "error": f"GitHub API error: {response.status_code}"}
                    
            except Exception as e:
                logger.error(f"Error validating GitHub token: {e}")
                return {"valid": False, "error": str(e)}
    
    async def check_repository_exists(self, repo_name: str, username: str) -> bool:
        """Check if a repository exists for the authenticated user"""
        async with httpx.AsyncClient() as client:
            try:
                response = await client.get(
                    f"{self.BASE_URL}/repos/{username}/{repo_name}",
                    headers=self.headers
                )
                
                return response.status_code == 200
                
            except Exception as e:
                logger.error(f"Error checking repository existence: {e}")
                return False
    
    async def create_repository(
        self, 
        repo_name: str, 
        description: str = "", 
        private: bool = False,
        auto_init: bool = False
    ) -> Dict[str, Any]:
        """Create a new GitHub repository"""
        
        # Get user info first
        user_info = await self.check_token_validity()
        if not user_info.get("valid"):
            raise GitHubAPIError("Invalid GitHub token", 401)
        
        username = user_info["username"]
        
        # Check if repository already exists
        if await self.check_repository_exists(repo_name, username):
            raise GitHubAPIError(f"Repository '{repo_name}' already exists", 409)
        
        async with httpx.AsyncClient() as client:
            try:
                payload = {
                    "name": repo_name,
                    "description": description,
                    "private": private,
                    "auto_init": auto_init,
                    "homepage": "",
                    "has_issues": True,
                    "has_projects": True,
                    "has_wiki": False,
                    "has_downloads": True
                }
                
                response = await client.post(
                    f"{self.BASE_URL}/user/repos",
                    headers=self.headers,
                    json=payload
                )
                
                if response.status_code == 201:
                    repo_data = response.json()
                    return {
                        "success": True,
                        "repo_url": repo_data["html_url"],
                        "clone_url": repo_data["clone_url"],
                        "ssh_url": repo_data["ssh_url"],
                        "git_url": repo_data["git_url"],
                        "name": repo_data["name"],
                        "full_name": repo_data["full_name"],
                        "repo_id": repo_data["id"],
                        "private": repo_data["private"],
                        "default_branch": repo_data["default_branch"] or "main"
                    }
                elif response.status_code == 422:
                    error_data = response.json()
                    if "errors" in error_data:
                        error_msg = "; ".join([err.get("message", "Unknown error") for err in error_data["errors"]])
                    else:
                        error_msg = error_data.get("message", "Repository creation failed")
                    raise GitHubAPIError(f"Repository creation failed: {error_msg}", 422)
                elif response.status_code == 401:
                    raise GitHubAPIError("GitHub authentication failed", 401)
                elif response.status_code == 403:
                    raise GitHubAPIError("GitHub access denied. Check token permissions", 403)
                else:
                    error_text = response.text
                    raise GitHubAPIError(f"GitHub API error: {response.status_code} - {error_text}", response.status_code)
                    
            except GitHubAPIError:
                raise
            except Exception as e:
                logger.error(f"Error creating GitHub repository: {e}")
                raise GitHubAPIError(f"Failed to create repository: {str(e)}")
    
    async def get_repository_info(self, username: str, repo_name: str) -> Optional[Dict[str, Any]]:
        """Get repository information including repository ID"""
        async with httpx.AsyncClient() as client:
            try:
                response = await client.get(
                    f"{self.BASE_URL}/repos/{username}/{repo_name}",
                    headers=self.headers
                )
                
                if response.status_code == 200:
                    repo_data = response.json()
                    return {
                        "repo_url": repo_data["html_url"],
                        "clone_url": repo_data["clone_url"],
                        "ssh_url": repo_data["ssh_url"],
                        "git_url": repo_data["git_url"],
                        "name": repo_data["name"],
                        "full_name": repo_data["full_name"],
                        "repo_id": repo_data["id"],
                        "private": repo_data["private"],
                        "default_branch": repo_data["default_branch"] or "main"
                    }
                else:
                    return None
                    
            except Exception as e:
                logger.error(f"Error getting repository info: {e}")
                return None
    
    async def get_user_repositories(self, per_page: int = 30, page: int = 1) -> Dict[str, Any]:
        """Get user's repositories"""
        async with httpx.AsyncClient() as client:
            try:
                response = await client.get(
                    f"{self.BASE_URL}/user/repos",
                    headers=self.headers,
                    params={
                        "per_page": per_page,
                        "page": page,
                        "sort": "updated",
                        "direction": "desc"
                    }
                )
                
                if response.status_code == 200:
                    return {
                        "success": True,
                        "repositories": response.json()
                    }
                else:
                    return {
                        "success": False,
                        "error": f"GitHub API error: {response.status_code}"
                    }
                    
            except Exception as e:
                logger.error(f"Error getting user repositories: {e}")
                return {
                    "success": False,
                    "error": str(e)
                }


# Utility functions
async def validate_github_token(token: str) -> Dict[str, Any]:
    """Validate a GitHub token and return user info"""
    github_service = GitHubService(token)
    return await github_service.check_token_validity()


async def check_repo_availability(token: str, repo_name: str) -> Dict[str, Any]:
    """Check if a repository name is available"""
    github_service = GitHubService(token)
    
    # First validate token and get username
    user_info = await github_service.check_token_validity()
    if not user_info.get("valid"):
        return {"available": False, "error": "Invalid GitHub token"}
    
    username = user_info["username"]
    exists = await github_service.check_repository_exists(repo_name, username)
    
    return {
        "available": not exists,
        "exists": exists,
        "username": username
    }





"""
Unified CLI facade

This module re-exports the public API for backward compatibility.
Implementations live in:
- Base/Utils: app/services/cli/base.py
- Providers: app/services/cli/adapters/*.py
- Manager: app/services/cli/manager.py
"""

from .base import BaseCLI, CLIType, MODEL_MAPPING, get_project_root, get_display_path
from .adapters import ClaudeCodeCLI, CursorAgentCLI, CodexCLI, QwenCLI, GeminiCLI
from .manager import UnifiedCLIManager

__all__ = [
    "BaseCLI",
    "CLIType",
    "MODEL_MAPPING",
    "get_project_root",
    "get_display_path",
    "ClaudeCodeCLI",
    "CursorAgentCLI",
    "CodexCLI",
    "QwenCLI",
    "GeminiCLI",
    "UnifiedCLIManager",
]



"""
Claudable Helper - Integration of Claudable services.

Provides easy access to Claudable's CLI adapters and services.
"""

# Import main CLI adapters for convenience
try:
    from .cli.adapters import (
        ClaudeCodeCLI,
        CursorAgentCLI,
        CodexCLI,
        QwenCLI,
        GeminiCLI,
    )
    __all__ = [
        "ClaudeCodeCLI",
        "CursorAgentCLI", 
        "CodexCLI",
        "QwenCLI",
        "GeminiCLI",
    ]
except ImportError as e:
    print(f"Warning: Could not import CLI adapters: {e}")
    __all__ = []

# Version info
__version__ = "extracted-from-claudable"



"""
CLI Session Manager for Multi-CLI Support
Handles session persistence and continuity across different CLI agents
"""
from typing import Dict, Optional, Any
from sqlalchemy.orm import Session
from ..models.projects import Project
from ..cli.base import CLIType


class CLISessionManager:
    """Manages CLI sessions across different AI agents"""
    
    def __init__(self, db: Session):
        self.db = db
        self._session_cache: Dict[str, Dict[CLIType, str]] = {}
    
    def get_session_id(self, project_id: str, cli_type: CLIType) -> Optional[str]:
        """Get existing session ID for a project and CLI type"""
        # Check cache first
        if project_id in self._session_cache:
            cached_session = self._session_cache[project_id].get(cli_type)
            if cached_session:
                return cached_session
        
        # Get from database
        project = self.db.get(Project, project_id)
        if not project:
            return None
        
        session_mapping = {
            CLIType.CLAUDE: project.active_claude_session_id,
            CLIType.CURSOR: project.active_cursor_session_id
        }
        
        session_id = session_mapping.get(cli_type)
        
        # Update cache
        if project_id not in self._session_cache:
            self._session_cache[project_id] = {}
        self._session_cache[project_id][cli_type] = session_id
        
        return session_id
    
    def set_session_id(self, project_id: str, cli_type: CLIType, session_id: str) -> bool:
        """Set session ID for a project and CLI type"""
        project = self.db.get(Project, project_id)
        if not project:
            return False
        
        # Update database
        update_mapping = {
            CLIType.CLAUDE: {"active_claude_session_id": session_id},
            CLIType.CURSOR: {"active_cursor_session_id": session_id}
        }
        
        update_data = update_mapping.get(cli_type)
        if not update_data:
            return False
        
        # Update project record
        for field, value in update_data.items():
            setattr(project, field, value)
        
        self.db.commit()
        
        # Update cache
        if project_id not in self._session_cache:
            self._session_cache[project_id] = {}
        self._session_cache[project_id][cli_type] = session_id
        
        from ..core.terminal_ui import ui
        ui.success(f"Set {cli_type.value} session ID for project {project_id}: {session_id}", "Session")
        return True
    
    def get_all_sessions(self, project_id: str) -> Dict[str, Optional[str]]:
        """Get all CLI session IDs for a project"""
        project = self.db.get(Project, project_id)
        if not project:
            return {}
        
        return {
            "claude": project.active_claude_session_id,
            "cursor": project.active_cursor_session_id
        }
    
    def clear_session_id(self, project_id: str, cli_type: CLIType) -> bool:
        """Clear session ID for a project and CLI type"""
        return self.set_session_id(project_id, cli_type, None)
    
    def clear_all_sessions(self, project_id: str) -> bool:
        """Clear all CLI session IDs for a project"""
        project = self.db.get(Project, project_id)
        if not project:
            return False
        
        project.active_claude_session_id = None
        project.active_cursor_session_id = None
        
        self.db.commit()
        
        # Clear cache
        if project_id in self._session_cache:
            del self._session_cache[project_id]
        
        from ..core.terminal_ui import ui
        ui.info(f"Cleared all CLI sessions for project {project_id}", "Session")
        return True
    
    def get_session_stats(self, project_id: str) -> Dict[str, Any]:
        """Get session statistics for a project"""
        from ..models.sessions import Session as ChatSession
        from sqlalchemy import func
        
        # Get session counts by CLI type
        session_stats = self.db.query(
            ChatSession.cli_type,
            func.count(ChatSession.id).label('count'),
            func.avg(ChatSession.duration_ms).label('avg_duration_ms'),
            func.sum(ChatSession.total_messages).label('total_messages'),
            func.max(ChatSession.started_at).label('last_used')
        ).filter(
            ChatSession.project_id == project_id
        ).group_by(ChatSession.cli_type).all()
        
        stats = {}
        for stat in session_stats:
            stats[stat.cli_type] = {
                "session_count": stat.count,
                "avg_duration_ms": int(stat.avg_duration_ms) if stat.avg_duration_ms else 0,
                "total_messages": stat.total_messages or 0,
                "last_used": stat.last_used.isoformat() if stat.last_used else None,
                "active_session_id": self.get_session_id(project_id, CLIType(stat.cli_type))
            }
        
        return stats
    
    def get_preferred_cli(self, project_id: str) -> Optional[CLIType]:
        """Get preferred CLI for a project"""
        project = self.db.get(Project, project_id)
        if not project:
            return None
        
        try:
            return CLIType(project.preferred_cli)
        except ValueError:
            return CLIType.CLAUDE  # Default fallback
    
    def set_preferred_cli(self, project_id: str, cli_type: CLIType, fallback_enabled: bool = True) -> bool:
        """Set preferred CLI for a project"""
        project = self.db.get(Project, project_id)
        if not project:
            return False
        
        project.preferred_cli = cli_type.value
        project.fallback_enabled = fallback_enabled
        self.db.commit()
        
        print(f"✅ [Session] Set preferred CLI for project {project_id}: {cli_type.value} (fallback: {fallback_enabled})")
        return True
    
    def is_fallback_enabled(self, project_id: str) -> bool:
        """Check if fallback is enabled for a project"""
        project = self.db.get(Project, project_id)
        if not project:
            return True  # Default to enabled
        
        return project.fallback_enabled
    
    def migrate_legacy_sessions(self, project_id: str) -> Dict[str, int]:
        """Migrate legacy Claude-only sessions to new CLI system"""
        from ..models.sessions import Session as ChatSession
        
        # Update sessions without cli_type
        updated_sessions = self.db.query(ChatSession).filter(
            ChatSession.project_id == project_id,
            ChatSession.cli_type == None
        ).update({"cli_type": CLIType.CLAUDE.value})
        
        # Update messages without cli_source where metadata suggests CLI type
        from ..models.messages import Message
        
        messages_updated = 0
        messages = self.db.query(Message).filter(
            Message.project_id == project_id,
            Message.cli_source == None,
            Message.metadata_json != None
        ).all()
        
        for message in messages:
            if message.metadata_json and "cli_type" in message.metadata_json:
                message.cli_source = message.metadata_json["cli_type"]
                messages_updated += 1
            else:
                message.cli_source = CLIType.CLAUDE.value  # Default to claude
                messages_updated += 1
        
        self.db.commit()
        
        migration_stats = {
            "sessions_updated": updated_sessions,
            "messages_updated": messages_updated
        }
        
        print(f"📊 [Migration] Project {project_id}: {migration_stats}")
        return migration_stats
    
    def cleanup_stale_sessions(self, project_id: str, days_threshold: int = 30) -> int:
        """Clean up old/stale CLI session IDs"""
        from datetime import datetime, timedelta
        from ..models.sessions import Session as ChatSession
        
        cutoff_date = datetime.utcnow() - timedelta(days=days_threshold)
        
        # Find sessions that haven't been used recently
        stale_sessions = self.db.query(ChatSession).filter(
            ChatSession.project_id == project_id,
            ChatSession.started_at < cutoff_date,
            ChatSession.status.in_(["completed", "failed"])
        ).all()
        
        # Clear session IDs for stale sessions
        cleared_count = 0
        for session in stale_sessions:
            if session.cli_type:
                try:
                    cli_type = CLIType(session.cli_type)
                    current_session_id = self.get_session_id(project_id, cli_type)
                    
                    # Only clear if it matches the stale session's claude_session_id
                    if current_session_id == session.claude_session_id:
                        self.clear_session_id(project_id, cli_type)
                        cleared_count += 1
                        
                except ValueError:
                    continue
        
        from ..core.terminal_ui import ui
        ui.info(f"Project {project_id}: Cleared {cleared_count} stale session IDs", "Cleanup")
        return cleared_count



"""Cursor Agent provider implementation.

Moved from unified_manager.py to a dedicated adapter module.
"""
from __future__ import annotations

import asyncio
import json
import os
import uuid
from datetime import datetime
from typing import Any, AsyncGenerator, Callable, Dict, List, Optional

from claudable_helper.models.messages import Message
from claudable_helper.core.terminal_ui import ui

from ..base import BaseCLI, CLIType, LineBuffer


class CursorAgentCLI(BaseCLI):
    """Cursor Agent CLI implementation with stream-json support and session continuity"""

    def __init__(self):
        super().__init__(CLIType.CURSOR)
        self._session_store = {}  # Simple in-memory session storage

    async def check_availability(self) -> Dict[str, Any]:
        """Check if Cursor Agent CLI is available"""
        try:
            # Check if cursor-agent is installed and working
            result = await asyncio.create_subprocess_shell(
                "cursor-agent -h",
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
            )
            stdout, stderr = await result.communicate()

            if result.returncode != 0:
                return {
                    "available": False,
                    "configured": False,
                    "error": (
                        "Cursor Agent CLI not installed or not working.\n\nTo install:\n"
                        "1. Install Cursor: curl https://cursor.com/install -fsS | bash\n"
                        "2. Login to Cursor: cursor-agent login\n3. Try running your prompt again"
                    ),
                }

            # Check if help output contains expected content
            help_output = stdout.decode() + stderr.decode()
            if "cursor-agent" not in help_output.lower():
                return {
                    "available": False,
                    "configured": False,
                    "error": (
                        "Cursor Agent CLI not responding correctly.\n\nPlease try:\n"
                        "1. Reinstall: curl https://cursor.com/install -fsS | bash\n"
                        "2. Login: cursor-agent login\n3. Check installation: cursor-agent -h"
                    ),
                }

            return {
                "available": True,
                "configured": True,
                "models": self.get_supported_models(),
                "default_models": ["gpt-5", "sonnet-4"],
            }
        except Exception as e:
            return {
                "available": False,
                "configured": False,
                "error": (
                    f"Failed to check Cursor Agent: {str(e)}\n\nTo install:\n"
                    "1. Install Cursor: curl https://cursor.com/install -fsS | bash\n"
                    "2. Login: cursor-agent login"
                ),
            }

    def _handle_cursor_stream_json(
        self, event: Dict[str, Any], project_path: str, session_id: str
    ) -> Optional[Message]:
        """Handle Cursor stream-json format (NDJSON events) to be compatible with Claude Code CLI output"""
        event_type = event.get("type")

        if event_type == "system":
            # System initialization event
            return Message(
                id=str(uuid.uuid4()),
                project_id=project_path,
                role="system",
                message_type="system",
                content=f"🔧 Cursor Agent initialized (Model: {event.get('model', 'unknown')})",
                metadata_json={
                    "cli_type": self.cli_type.value,
                    "event_type": "system",
                    "cwd": event.get("cwd"),
                    "api_key_source": event.get("apiKeySource"),
                    "original_event": event,
                    "hidden_from_ui": True,  # Hide system init messages
                },
                session_id=session_id,
                created_at=datetime.utcnow(),
            )

        elif event_type == "user":
            # Cursor echoes back the user's prompt. Suppress it to avoid duplicates.
            return None

        elif event_type == "assistant":
            # Assistant response event (text delta)
            message_content = event.get("message", {}).get("content", [])
            content = ""

            if message_content and isinstance(message_content, list):
                for part in message_content:
                    if part.get("type") == "text":
                        content += part.get("text", "")

            if content:
                return Message(
                    id=str(uuid.uuid4()),
                    project_id=project_path,
                    role="assistant",
                    message_type="chat",
                    content=content,
                    metadata_json={
                        "cli_type": self.cli_type.value,
                        "event_type": "assistant",
                        "original_event": event,
                    },
                    session_id=session_id,
                    created_at=datetime.utcnow(),
                )

        elif event_type == "tool_call":
            subtype = event.get("subtype")
            tool_call_data = event.get("tool_call", {})
            if not tool_call_data:
                return None

            tool_name_raw = next(iter(tool_call_data), None)
            if not tool_name_raw:
                return None

            # Normalize tool name: lsToolCall -> ls
            tool_name = tool_name_raw.replace("ToolCall", "")

            if subtype == "started":
                tool_input = tool_call_data[tool_name_raw].get("args", {})
                summary = self._create_tool_summary(tool_name, tool_input)

                return Message(
                    id=str(uuid.uuid4()),
                    project_id=project_path,
                    role="assistant",
                    message_type="chat",
                    content=summary,
                    metadata_json={
                        "cli_type": self.cli_type.value,
                        "event_type": "tool_call_started",
                        "tool_name": tool_name,
                        "tool_input": tool_input,
                        "original_event": event,
                    },
                    session_id=session_id,
                    created_at=datetime.utcnow(),
                )

            elif subtype == "completed":
                result = tool_call_data[tool_name_raw].get("result", {})
                content = ""
                if "success" in result:
                    content = json.dumps(result["success"])
                elif "error" in result:
                    content = json.dumps(result["error"])

                return Message(
                    id=str(uuid.uuid4()),
                    project_id=project_path,
                    role="system",
                    message_type="tool_result",
                    content=content,
                    metadata_json={
                        "cli_type": self.cli_type.value,
                        "original_format": event,
                        "tool_name": tool_name,
                        "hidden_from_ui": True,
                    },
                    session_id=session_id,
                    created_at=datetime.utcnow(),
                )

        elif event_type == "result":
            # Final result event
            duration = event.get("duration_ms", 0)
            result_text = event.get("result", "")

            if result_text:
                return Message(
                    id=str(uuid.uuid4()),
                    project_id=project_path,
                    role="system",
                    message_type="system",
                    content=(
                        f"Execution completed in {duration}ms. Final result: {result_text}"
                    ),
                    metadata_json={
                        "cli_type": self.cli_type.value,
                        "event_type": "result",
                        "duration_ms": duration,
                        "original_event": event,
                        "hidden_from_ui": True,
                    },
                    session_id=session_id,
                    created_at=datetime.utcnow(),
                )

        return None


    async def execute_with_streaming(
        self,
        instruction: str,
        project_path: str,
        session_id: Optional[str] = None,
        log_callback: Optional[Callable[[str], Any]] = None,
        images: Optional[List[Dict[str, Any]]] = None,
        model: Optional[str] = None,
        is_initial_prompt: bool = False,
    ) -> AsyncGenerator[Message, None]:
        """Execute Cursor Agent CLI with stream-json format and session continuity"""
        # Skip AGENTS.md creation - removed for MCP server usage

        # Extract project ID - simplified path handling
        project_id = os.path.basename(project_path)

        stored_session_id = await self.get_session_id(project_id)

        cmd = [
            "cursor-agent",
            "--force",
            "-p",
            instruction,
            "--output-format",
            "stream-json",  # Use stream-json format
        ]

        # Add session resume if available (prefer stored session over parameter)
        active_session_id = stored_session_id or session_id
        if active_session_id:
            cmd.extend(["--resume", active_session_id])
            print(f"🔗 [Cursor] Resuming session: {active_session_id}")

        # Add API key if available
        if os.getenv("CURSOR_API_KEY"):
            cmd.extend(["--api-key", os.getenv("CURSOR_API_KEY")])

        # Add model - prioritize parameter over environment variable
        cli_model = self._get_cli_model_name(model) or os.getenv("CURSOR_MODEL")
        if cli_model:
            cmd.extend(["--model", cli_model])
            print(f"🔧 [Cursor] Using model: {cli_model}")

        # Use the provided project path directly
        project_repo_path = project_path

        try:
            process = await asyncio.create_subprocess_exec(
                *cmd,
                stdin=asyncio.subprocess.DEVNULL,  # Explicitly close stdin
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
                cwd=project_repo_path,
            )

            # Wrap stdout with LineBuffer for large NDJSON handling
            reader = LineBuffer(process.stdout)

            # Start stderr reader task
            stderr_task = asyncio.create_task(self._drain_stderr(process.stderr))

            cursor_session_id = None
            assistant_message_buffer = ""
            result_received = False  # Track if we received result event

            # Process streaming events with timeout to prevent hanging
            READLINE_TIMEOUT = 300  # 5 minutes timeout for readline operations
            consecutive_timeouts = 0
            max_consecutive_timeouts = 3  # Allow up to 3 consecutive timeouts before giving up

            while True:
                try:
                    # Add timeout to readline to prevent indefinite hanging
                    line = await asyncio.wait_for(reader.readline(), timeout=READLINE_TIMEOUT)
                    consecutive_timeouts = 0  # Reset timeout counter on successful read

                    if not line:
                        break
                except asyncio.TimeoutError:
                    consecutive_timeouts += 1
                    ui.warning(f"Readline timeout #{consecutive_timeouts} - process may be idle", "Cursor")

                    # Check if process is still alive
                    if process.returncode is not None:
                        ui.info("Process has terminated, ending stream", "Cursor")
                        break

                    # If we've had too many consecutive timeouts, assume process is hung
                    if consecutive_timeouts >= max_consecutive_timeouts:
                        ui.error(f"Process appears hung after {consecutive_timeouts} timeouts, terminating", "Cursor")
                        try:
                            process.terminate()
                            await asyncio.wait_for(process.wait(), timeout=10)
                        except asyncio.TimeoutError:
                            ui.warning("Process did not terminate gracefully, killing", "Cursor")
                            process.kill()
                        break

                    # Continue to next iteration to try reading again
                    continue
                line_str = line.decode().strip()
                if not line_str:
                    continue

                try:
                    # Parse NDJSON event
                    event = json.loads(line_str)

                    event_type = event.get("type")

                    # Priority: Extract session ID from type: "result" event (most reliable)
                    if event_type == "result":
                        print(f"🔍 [Cursor] Result event received: {event}")

                        # Extract session ID if not already found
                        if not cursor_session_id:
                            session_id_from_result = event.get("session_id")
                            if session_id_from_result:
                                cursor_session_id = session_id_from_result
                                await self.set_session_id(project_id, cursor_session_id)
                                print(
                                    f"💾 [Cursor] Session ID extracted from result event: {cursor_session_id}"
                                )

                        # Emit result message for MCP server
                        result_text = event.get("result", "")
                        yield Message(
                            id=str(uuid.uuid4()),
                            project_id=project_path,
                            role="assistant",
                            message_type="result",
                            content=result_text,
                            metadata_json={
                                "cli_type": "cursor",
                                "event_type": "result",
                                "session_id": cursor_session_id,
                            },
                            session_id=session_id,
                            created_at=datetime.utcnow(),
                        )

                        # Mark that we received result event
                        result_received = True

                    # Extract session ID from various event types
                    if not cursor_session_id:
                        # Try to extract session ID from any event that contains it
                        potential_session_id = (
                            event.get("sessionId")
                            or event.get("chatId")
                            or event.get("session_id")
                            or event.get("chat_id")
                            or event.get("threadId")
                            or event.get("thread_id")
                        )

                        # Also check in nested structures
                        if not potential_session_id and isinstance(
                            event.get("message"), dict
                        ):
                            potential_session_id = (
                                event["message"].get("sessionId")
                                or event["message"].get("chatId")
                                or event["message"].get("session_id")
                                or event["message"].get("chat_id")
                            )

                        if potential_session_id and potential_session_id != active_session_id:
                            cursor_session_id = potential_session_id
                            await self.set_session_id(project_id, cursor_session_id)
                            print(
                                f"💾 [Cursor] Updated session ID for project {project_id}: {cursor_session_id}"
                            )
                            print(f"   Previous: {active_session_id}")
                            print(f"   New: {cursor_session_id}")

                    # If we receive a non-assistant message, flush the buffer first
                    if event.get("type") != "assistant" and assistant_message_buffer:
                        yield Message(
                            id=str(uuid.uuid4()),
                            project_id=project_path,
                            role="assistant",
                            message_type="chat",
                            content=assistant_message_buffer,
                            metadata_json={
                                "cli_type": "cursor",
                                "event_type": "assistant_aggregated",
                            },
                            session_id=session_id,
                            created_at=datetime.utcnow(),
                        )
                        assistant_message_buffer = ""

                    # Process the event
                    message = self._handle_cursor_stream_json(
                        event, project_path, session_id
                    )

                    if message:
                        if message.role == "assistant" and message.message_type == "chat":
                            assistant_message_buffer += message.content
                        else:
                            if log_callback:
                                await log_callback(f"📝 [Cursor] {message.content}")
                            yield message

                    # ★ CRITICAL: Break after result event to end streaming
                    if result_received:
                        print(
                            f"🏁 [Cursor] Result event received, terminating stream early"
                        )
                        try:
                            process.terminate()
                            print(f"🔪 [Cursor] Process terminated")
                        except Exception as e:
                            print(f"⚠️ [Cursor] Failed to terminate process: {e}")
                        break

                except json.JSONDecodeError as e:
                    # Handle malformed JSON
                    print(f"⚠️ [Cursor] JSON decode error: {e}")
                    print(f"⚠️ [Cursor] Raw line: {line_str}")

                    # Still yield as raw output
                    message = Message(
                        id=str(uuid.uuid4()),
                        project_id=project_path,
                        role="assistant",
                        message_type="chat",
                        content=line_str,
                        metadata_json={
                            "cli_type": "cursor",
                            "raw_output": line_str,
                            "parse_error": str(e),
                        },
                        session_id=session_id,
                        created_at=datetime.utcnow(),
                    )
                    yield message

            # Flush any remaining content in the buffer
            if assistant_message_buffer:
                yield Message(
                    id=str(uuid.uuid4()),
                    project_id=project_path,
                    role="assistant",
                    message_type="chat",
                    content=assistant_message_buffer,
                    metadata_json={
                        "cli_type": "cursor",
                        "event_type": "assistant_aggregated",
                    },
                    session_id=session_id,
                    created_at=datetime.utcnow(),
                )

        except asyncio.CancelledError:
            # Handle cancellation gracefully
            print(f"🔄 [Cursor] Operation cancelled, cleaning up process")
            raise
        except Exception as e:
            print(f"❌ [Cursor] Error during execution: {e}")
            raise
        except FileNotFoundError:
            error_msg = (
                "❌ Cursor Agent CLI not found. Please install with: curl https://cursor.com/install -fsS | bash"
            )
            yield Message(
                id=str(uuid.uuid4()),
                project_id=project_path,
                role="assistant",
                message_type="error",
                content=error_msg,
                metadata_json={"error": "cli_not_found", "cli_type": "cursor"},
                session_id=session_id,
                created_at=datetime.utcnow(),
            )
        except asyncio.CancelledError:
            # Propagate cancellation
            raise
        except Exception as e:
            error_msg = f"❌ Cursor Agent execution failed: {str(e)}"
            yield Message(
                id=str(uuid.uuid4()),
                project_id=project_path,
                role="assistant",
                message_type="error",
                content=error_msg,
                metadata_json={
                    "error": "execution_failed",
                    "cli_type": "cursor",
                    "exception": str(e),
                },
                session_id=session_id,
                created_at=datetime.utcnow(),
            )
        finally:
            # Always clean up process and tasks
            if 'process' in locals():
                stderr_task_var = locals().get('stderr_task')
                await self._cleanup_cursor_process(process, stderr_task_var)

    async def get_session_id(self, project_id: str) -> Optional[str]:
        """Get stored session ID for project"""
        return self._session_store.get(project_id)

    async def set_session_id(self, project_id: str, session_id: str) -> None:
        """Store session ID for project in memory"""
        self._session_store[project_id] = session_id
        print(
            f"💾 [Cursor] Session ID stored for project {project_id}: {session_id}"
        )

    async def _drain_stderr(self, stderr) -> None:
        """Background task to drain stderr to prevent blocking."""
        if not stderr:
            return

        try:
            stderr_reader = LineBuffer(stderr)
            while True:
                line = await stderr_reader.readline()
                if not line:
                    break
                # Optionally log stderr for debugging
                line_str = line.decode().strip()
                if line_str:
                    print(f"🔍 [Cursor] stderr: {line_str}")
        except asyncio.CancelledError:
            pass
        except Exception:
            pass

    async def _cleanup_cursor_process(self, process, stderr_task) -> None:
        """Clean up cursor process and associated tasks."""
        try:
            # Cancel stderr task
            if stderr_task and not stderr_task.done():
                stderr_task.cancel()
                try:
                    await stderr_task
                except asyncio.CancelledError:
                    pass

            # Gracefully terminate process
            if process and process.returncode is None:
                print(f"🔄 [Cursor] Terminating process gracefully")
                process.terminate()
                try:
                    # Wait up to 5 seconds for graceful termination
                    await asyncio.wait_for(process.wait(), timeout=5.0)
                    print(f"🔪 [Cursor] Process terminated gracefully")
                except asyncio.TimeoutError:
                    print(f"⚠️ [Cursor] Process didn't terminate gracefully, killing")
                    process.kill()
                    await process.wait()
                    print(f"🔪 [Cursor] Process killed")
        except Exception as e:
            print(f"⚠️ [Cursor] Error during cleanup: {e}")


__all__ = ["CursorAgentCLI"]



"""
Base abstractions and shared utilities for CLI providers.

This module defines a precise, minimal adapter contract (BaseCLI) and common
helpers so that adding a new provider remains consistent and easy.
"""
from __future__ import annotations

import os
import uuid
from abc import ABC, abstractmethod
from contextlib import asynccontextmanager
from datetime import datetime
from enum import Enum
from typing import Any, AsyncGenerator, Callable, Dict, List, Optional

from ..models.messages import Message


class LineBuffer:
    """Dead simple line buffer for async streams.

    Handles arbitrary line lengths by reading in chunks until newline found.
    Solves asyncio StreamReader 64KB buffer limit for large NDJSON responses.
    """

    def __init__(self, stream):
        self.stream = stream
        self.buffer = b''

    async def readline(self):
        """Read until newline, handling arbitrary line lengths."""
        while b'\n' not in self.buffer:
            chunk = await self.stream.read(8192)  # 8KB chunks
            if not chunk:
                # EOF
                line = self.buffer
                self.buffer = b''
                return line
            self.buffer += chunk

        # Split on first newline
        line, self.buffer = self.buffer.split(b'\n', 1)
        return line + b'\n'


def get_project_root() -> str:
    """Return project root directory using relative path navigation.

    This function intentionally mirrors the logic previously embedded in
    unified_manager.py so imports remain stable after refactor.
    """
    current_file_dir = os.path.dirname(os.path.abspath(__file__))
    # base.py is in: app/services/cli/
    # Navigate: cli -> services -> app -> api -> apps -> project-root
    project_root = os.path.join(current_file_dir, "..", "..", "..", "..", "..")
    return os.path.abspath(project_root)


def get_display_path(file_path: str) -> str:
    """Convert absolute path to a shorter display path scoped to the project.

    - Strips the project root prefix when present
    - Compacts repo-specific prefixes (e.g., data/projects -> …/)
    """
    try:
        project_root = get_project_root()
        if file_path.startswith(project_root):
            display_path = file_path.replace(project_root + "/", "")
            return display_path.replace("data/projects/", "…/")
    except Exception:
        pass
    return file_path


# Model mapping from unified names to CLI-specific names
MODEL_MAPPING: Dict[str, Dict[str, str]] = {
    "claude": {
        "opus-4.1": "claude-opus-4-1-20250805",
        "sonnet-4": "claude-sonnet-4-20250514",
        "opus-4": "claude-opus-4-20250514",
        "haiku-3.5": "claude-3-5-haiku-20241022",
        # Handle claude-prefixed model names
        "claude-sonnet-4": "claude-sonnet-4-20250514",
        "claude-opus-4.1": "claude-opus-4-1-20250805",
        "claude-opus-4": "claude-opus-4-20250514",
        "claude-haiku-3.5": "claude-3-5-haiku-20241022",
        # Support direct full model names
        "claude-opus-4-1-20250805": "claude-opus-4-1-20250805",
        "claude-sonnet-4-20250514": "claude-sonnet-4-20250514",
        "claude-opus-4-20250514": "claude-opus-4-20250514",
        "claude-3-5-haiku-20241022": "claude-3-5-haiku-20241022",
    },
    "cursor": {
        "gpt-5": "gpt-5",
        "sonnet-4": "sonnet-4",
        "opus-4.1": "opus-4.1",
        "sonnet-4-thinking": "sonnet-4-thinking",
        # Handle mapping from unified Claude model names
        "claude-sonnet-4": "sonnet-4",
        "claude-opus-4.1": "opus-4.1",
        "claude-sonnet-4-20250514": "sonnet-4",
        "claude-opus-4-1-20250805": "opus-4.1",
    },
    "codex": {
        "gpt-5": "gpt-5",
        "gpt-4o": "gpt-4o",
        "gpt-4o-mini": "gpt-4o-mini",
        "o1-preview": "o1-preview",
        "o1-mini": "o1-mini",
        "claude-3.5-sonnet": "claude-3.5-sonnet",
        "claude-3-haiku": "claude-3-haiku",
        # Handle unified model names
        "sonnet-4": "claude-3.5-sonnet",
        "claude-sonnet-4": "claude-3.5-sonnet",
        "haiku-3.5": "claude-3-haiku",
        "claude-haiku-3.5": "claude-3-haiku",
    },
    "qwen": {
        # Unified name → provider mapping
        "qwen3-coder-plus": "qwen-coder",
        "Qwen3 Coder Plus": "qwen-coder",
        # Allow direct
        "qwen-coder": "qwen-coder",
    },
    "gemini": {
        "gemini-2.5-pro": "gemini-2.5-pro",
        "gemini-2.5-flash": "gemini-2.5-flash",
    },
}


class CLIType(str, Enum):
    """Provider key used across the manager and adapters."""

    CLAUDE = "claude"
    CURSOR = "cursor"
    CODEX = "codex"
    QWEN = "qwen"
    GEMINI = "gemini"


class AdapterSession:
    """Async context manager for adapter lifecycle management.

    Ensures proper setup and teardown of subprocess/client resources.
    Inspired by TinyAgent's MCP client pattern.
    """

    def __init__(self, client, start_method=None, stop_method=None):
        """Initialize session with client and lifecycle methods.

        Args:
            client: The client object to manage (e.g., _ACPClient)
            start_method: Async method to call on enter (default: client.start)
            stop_method: Async method to call on exit (default: client.stop)
        """
        self.client = client
        self.start_method = start_method or (client.start if hasattr(client, 'start') else None)
        self.stop_method = stop_method or (client.stop if hasattr(client, 'stop') else None)
        self.started = False

    async def __aenter__(self):
        """Start the client/process and register handlers."""
        if self.start_method:
            await self.start_method()
            self.started = True
        return self.client

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """Clean up: unregister handlers, cancel tasks, terminate process."""
        if self.started and self.stop_method:
            try:
                await self.stop_method()
            except Exception:
                # Best effort cleanup
                pass
        return False  # Don't suppress exceptions


@asynccontextmanager
async def adapter_session(client, start_method=None, stop_method=None):
    """Create an async context manager for adapter lifecycle.

    Usage:
        async with adapter_session(client) as c:
            result = await c.request("method", params)
    """
    session = AdapterSession(client, start_method, stop_method)
    try:
        await session.__aenter__()
        yield session.client
    finally:
        await session.__aexit__(None, None, None)


class BaseCLI(ABC):
    """Abstract adapter contract for CLI providers.

    Subclasses must implement availability checks, streaming execution, and
    session persistence. Common utilities (model mapping, content parsing,
    tool summaries) are provided here for reuse.
    """

    def __init__(self, cli_type: CLIType):
        self.cli_type = cli_type

    # ---- Mandatory adapter interface ------------------------------------
    @abstractmethod
    async def check_availability(self) -> Dict[str, Any]:
        """Return provider availability/configuration status.

        Expected keys in the returned dict used by the manager:
        - available: bool
        - configured: bool
        - models/default_models (optional): List[str]
        - error (optional): str
        """

    @abstractmethod
    async def execute_with_streaming(
        self,
        instruction: str,
        project_path: str,
        session_id: Optional[str] = None,
        log_callback: Optional[Callable[[str], Any]] = None,
        images: Optional[List[Dict[str, Any]]] = None,
        model: Optional[str] = None,
        is_initial_prompt: bool = False,
    ) -> AsyncGenerator[Message, None]:
        """Execute an instruction and yield `Message` objects in real time."""

    @abstractmethod
    async def get_session_id(self, project_id: str) -> Optional[str]:
        """Return the active session ID for a project, if any."""

    @abstractmethod
    async def set_session_id(self, project_id: str, session_id: str) -> None:
        """Persist the active session ID for a project."""

    # ---- Common helpers (available to adapters) --------------------------
    def _get_cli_model_name(self, model: Optional[str]) -> Optional[str]:
        """Translate unified model name to provider-specific model name.

        If the input is already a provider name or mapping fails, return as-is.
        """
        if not model:
            return None

        from ..core.terminal_ui import ui

        ui.debug(f"Input model: '{model}' for CLI: {self.cli_type.value}", "Model")
        cli_models = MODEL_MAPPING.get(self.cli_type.value, {})

        # Try exact mapping
        if model in cli_models:
            mapped_model = cli_models[model]
            ui.info(
                f"Mapped '{model}' to '{mapped_model}' for {self.cli_type.value}", "Model"
            )
            return mapped_model

        # Already a provider-specific name
        if model in cli_models.values():
            ui.info(
                f"Using direct model name '{model}' for {self.cli_type.value}", "Model"
            )
            return model

        # Debug available models
        available_models = list(cli_models.keys())
        ui.warning(
            f"Model '{model}' not found in mapping for {self.cli_type.value}", "Model"
        )
        ui.debug(
            f"Available models for {self.cli_type.value}: {available_models}", "Model"
        )
        ui.warning(f"Using model as-is: '{model}'", "Model")
        return model

    def get_supported_models(self) -> List[str]:
        cli_models = MODEL_MAPPING.get(self.cli_type.value, {})
        return list(cli_models.keys()) + list(cli_models.values())

    def is_model_supported(self, model: str) -> bool:
        return (
            model in self.get_supported_models()
            or model in MODEL_MAPPING.get(self.cli_type.value, {}).values()
        )

    def parse_message_data(self, data: Dict[str, Any], project_id: str, session_id: str) -> Message:
        """Normalize provider-specific message payload to our `Message`."""
        return Message(
            id=str(uuid.uuid4()),
            project_id=project_id,
            role=self._normalize_role(data.get("role", "assistant")),
            message_type="chat",
            content=self._extract_content(data),
            metadata_json={
                **data,
                "cli_type": self.cli_type.value,
                "original_format": data,
            },
            session_id=session_id,
            created_at=datetime.utcnow(),
        )

    def _normalize_role(self, role: str) -> str:
        role_mapping = {
            "model": "assistant",
            "ai": "assistant",
            "human": "user",
            "bot": "assistant",
        }
        return role_mapping.get(role.lower(), role.lower())

    def _extract_content(self, data: Dict[str, Any]) -> str:
        """Extract best-effort text content from various provider formats."""
        # Claude content array
        if "content" in data and isinstance(data["content"], list):
            content = ""
            for item in data["content"]:
                if item.get("type") == "text":
                    content += item.get("text", "")
                elif item.get("type") == "tool_use":
                    tool_name = item.get("name", "Unknown")
                    tool_input = item.get("input", {})
                    summary = self._create_tool_summary(tool_name, tool_input)
                    content += f"{summary}\n"
            return content

        # Simple text
        elif "content" in data:
            return str(data["content"])

        # Gemini parts
        elif "parts" in data:
            content = ""
            for part in data["parts"]:
                if "text" in part:
                    content += part.get("text", "")
                elif "functionCall" in part:
                    func_call = part["functionCall"]
                    tool_name = func_call.get("name", "Unknown")
                    tool_input = func_call.get("args", {})
                    summary = self._create_tool_summary(tool_name, tool_input)
                    content += f"{summary}\n"
            return content

        # OpenAI/Codex choices
        elif "choices" in data and data["choices"]:
            choice = data["choices"][0]
            if "message" in choice:
                return choice["message"].get("content", "")
            elif "text" in choice:
                return choice.get("text", "")

        # Direct text fields
        elif "text" in data:
            return str(data["text"])
        elif "message" in data:
            if isinstance(data["message"], dict):
                return self._extract_content(data["message"])
            return str(data["message"])

        # Generic response field
        elif "response" in data:
            return str(data["response"])

        # Delta streaming
        elif "delta" in data and "content" in data["delta"]:
            return str(data["delta"]["content"])

        # Fallback
        else:
            return str(data)

    def _normalize_tool_name(self, tool_name: str) -> str:
        """Normalize tool names across providers to a unified label."""
        key = (tool_name or "").strip()
        key_lower = key.replace(" ", "").lower()
        tool_mapping = {
            # File operations
            "read_file": "Read",
            "read": "Read",
            "write_file": "Write",
            "write": "Write",
            "edit_file": "Edit",
            "replace": "Edit",
            "edit": "Edit",
            "delete": "Delete",
            # Qwen/Gemini variants (CamelCase / spaced)
            "readfile": "Read",
            "readfolder": "LS",
            "readmanyfiles": "Read",
            "writefile": "Write",
            "findfiles": "Glob",
            "savememory": "SaveMemory",
            "save memory": "SaveMemory",
            "searchtext": "Grep",
            # Terminal operations
            "shell": "Bash",
            "run_terminal_command": "Bash",
            # Search operations
            "search_file_content": "Grep",
            "codebase_search": "Grep",
            "grep": "Grep",
            "find_files": "Glob",
            "glob": "Glob",
            "list_directory": "LS",
            "list_dir": "LS",
            "ls": "LS",
            "semSearch": "SemSearch",
            # Web operations
            "google_web_search": "WebSearch",
            "web_search": "WebSearch",
            "googlesearch": "WebSearch",
            "web_fetch": "WebFetch",
            "fetch": "WebFetch",
            # Task/Memory operations
            "save_memory": "SaveMemory",
            # Codex operations
            "exec_command": "Bash",
            "apply_patch": "Edit",
            "mcp_tool_call": "MCPTool",
            # Generic simple names
            "search": "Grep",
        }
        return tool_mapping.get(tool_name, tool_mapping.get(key_lower, key))

    def _get_clean_tool_display(self, tool_name: str, tool_input: Dict[str, Any]) -> str:
        """Return a concise, Claude-like tool usage display line."""
        normalized_name = self._normalize_tool_name(tool_name)

        if normalized_name == "Read":
            file_path = (
                tool_input.get("file_path")
                or tool_input.get("path")
                or tool_input.get("file", "")
            )
            if file_path:
                filename = file_path.split("/")[-1]
                return f"Reading {filename}"
            return "Reading file"
        elif normalized_name == "Write":
            file_path = (
                tool_input.get("file_path")
                or tool_input.get("path")
                or tool_input.get("file", "")
            )
            if file_path:
                filename = file_path.split("/")[-1]
                return f"Writing {filename}"
            return "Writing file"
        elif normalized_name == "Edit":
            file_path = (
                tool_input.get("file_path")
                or tool_input.get("path")
                or tool_input.get("file", "")
            )
            if file_path:
                filename = file_path.split("/")[-1]
                return f"Editing {filename}"
            return "Editing file"
        elif normalized_name == "Bash":
            command = (
                tool_input.get("command")
                or tool_input.get("cmd")
                or tool_input.get("script", "")
            )
            if command:
                cmd_display = command.split()[0] if command.split() else command
                return f"Running {cmd_display}"
            return "Running command"
        elif normalized_name == "LS":
            return "Listing directory"
        elif normalized_name == "TodoWrite":
            return "Planning next steps"
        elif normalized_name == "WebSearch":
            query = tool_input.get("query", "")
            if query:
                return f"Searching: {query[:50]}..."
            return "Web search"
        elif normalized_name == "WebFetch":
            url = tool_input.get("url", "")
            if url:
                domain = (
                    url.split("//")[-1].split("/")[0]
                    if "//" in url
                    else url.split("/")[0]
                )
                return f"Fetching from {domain}"
            return "Fetching web content"
        else:
            return f"Using {tool_name}"

    def _create_tool_summary(self, tool_name: str, tool_input: Dict[str, Any]) -> str:
        """Create a visual markdown summary for tool usage.

        NOTE: Special-cases Codex `apply_patch` to render one-line summaries per
        file similar to Claude Code.
        """
        # Handle apply_patch BEFORE normalization to avoid confusion with Edit
        if tool_name == "apply_patch":
            changes = tool_input.get("changes", {})
            if isinstance(changes, dict) and changes:
                if len(changes) == 1:
                    path, change = next(iter(changes.items()))
                    filename = str(path).split("/")[-1]
                    if isinstance(change, dict):
                        if "add" in change:
                            return f"**Write** `{filename}`"
                        elif "delete" in change:
                            return f"**Delete** `{filename}`"
                        elif "update" in change:
                            upd = change.get("update") or {}
                            move_path = upd.get("move_path")
                            if move_path:
                                new_filename = move_path.split("/")[-1]
                                return f"**Rename** `{filename}` → `{new_filename}`"
                            else:
                                return f"**Edit** `{filename}`"
                        else:
                            return f"**Edit** `{filename}`"
                    else:
                        return f"**Edit** `{filename}`"
                else:
                    file_summaries: List[str] = []
                    for raw_path, change in list(changes.items())[:3]:  # max 3 files
                        path = str(raw_path)
                        filename = path.split("/")[-1]
                        if isinstance(change, dict):
                            if "add" in change:
                                file_summaries.append(f"• **Write** `{filename}`")
                            elif "delete" in change:
                                file_summaries.append(f"• **Delete** `{filename}`")
                            elif "update" in change:
                                upd = change.get("update") or {}
                                move_path = upd.get("move_path")
                                if move_path:
                                    new_filename = move_path.split("/")[-1]
                                    file_summaries.append(
                                        f"• **Rename** `{filename}` → `{new_filename}`"
                                    )
                                else:
                                    file_summaries.append(f"• **Edit** `{filename}`")
                            else:
                                file_summaries.append(f"• **Edit** `{filename}`")
                        else:
                            file_summaries.append(f"• **Edit** `{filename}`")

                    result = "\n".join(file_summaries)
                    if len(changes) > 3:
                        result += f"\n• ... +{len(changes) - 3} more files"
                    return result
            return "**ApplyPatch** `files`"

        # Normalize name after handling apply_patch
        normalized_name = self._normalize_tool_name(tool_name)

        if normalized_name == "Edit":
            file_path = (
                tool_input.get("file_path")
                or tool_input.get("path")
                or tool_input.get("file", "")
            )
            if file_path:
                display_path = get_display_path(file_path)
                if len(display_path) > 40:
                    display_path = "…/" + "/".join(display_path.split("/")[-2:])
                return f"**Edit** `{display_path}`"
            return "**Edit** `file`"
        elif normalized_name == "Read":
            file_path = (
                tool_input.get("file_path")
                or tool_input.get("path")
                or tool_input.get("file", "")
            )
            if file_path:
                display_path = get_display_path(file_path)
                if len(display_path) > 40:
                    display_path = "…/" + "/".join(display_path.split("/")[-2:])
                return f"**Read** `{display_path}`"
            return "**Read** `file`"
        elif normalized_name == "Bash":
            command = (
                tool_input.get("command")
                or tool_input.get("cmd")
                or tool_input.get("script", "")
            )
            if command:
                display_cmd = command[:40] + "..." if len(command) > 40 else command
                return f"**Bash** `{display_cmd}`"
            return "**Bash** `command`"
        elif normalized_name == "TodoWrite":
            return "`Planning for next moves...`"
        elif normalized_name == "SaveMemory":
            fact = tool_input.get("fact", "")
            if fact:
                return f"**SaveMemory** `{fact[:40]}{'...' if len(fact) > 40 else ''}`"
            return "**SaveMemory** `storing information`"
        elif normalized_name == "Grep":
            pattern = (
                tool_input.get("pattern")
                or tool_input.get("query")
                or tool_input.get("search", "")
            )
            path = (
                tool_input.get("path")
                or tool_input.get("file")
                or tool_input.get("directory", "")
            )
            if pattern:
                if path:
                    display_path = get_display_path(path)
                    return f"**Search** `{pattern}` in `{display_path}`"
                return f"**Search** `{pattern}`"
            return "**Search** `pattern`"
        elif normalized_name == "Glob":
            if tool_name == "find_files":
                name = tool_input.get("name", "")
                if name:
                    return f"**Glob** `{name}`"
                return "**Glob** `finding files`"
            pattern = tool_input.get("pattern", "") or tool_input.get(
                "globPattern", ""
            )
            if pattern:
                return f"**Glob** `{pattern}`"
            return "**Glob** `pattern`"
        elif normalized_name == "Write":
            file_path = (
                tool_input.get("file_path")
                or tool_input.get("path")
                or tool_input.get("file", "")
            )
            if file_path:
                display_path = get_display_path(file_path)
                if len(display_path) > 40:
                    display_path = "…/" + "/".join(display_path.split("/")[-2:])
                return f"**Write** `{display_path}`"
            return "**Write** `file`"
        elif normalized_name == "MultiEdit":
            file_path = (
                tool_input.get("file_path")
                or tool_input.get("path")
                or tool_input.get("file", "")
            )
            if file_path:
                display_path = get_display_path(file_path)
                if len(display_path) > 40:
                    display_path = "…/" + "/".join(display_path.split("/")[-2:])
                return f"🔧 **MultiEdit** `{display_path}`"
            return "🔧 **MultiEdit** `file`"
        elif normalized_name == "LS":
            path = (
                tool_input.get("path")
                or tool_input.get("directory")
                or tool_input.get("dir", "")
            )
            if path:
                display_path = get_display_path(path)
                if len(display_path) > 40:
                    display_path = "…/" + display_path[-37:]
                return f"📁 **LS** `{display_path}`"
            return "📁 **LS** `directory`"
        elif normalized_name == "WebFetch":
            url = tool_input.get("url", "")
            if url:
                domain = (
                    url.split("//")[-1].split("/")[0]
                    if "//" in url
                    else url.split("/")[0]
                )
                return f"**WebFetch** [{domain}]({url})"
            return "**WebFetch** `url`"
        elif normalized_name == "WebSearch":
            query = tool_input.get("query") or tool_input.get("search_query", "")
            query = tool_input.get("query", "")
            if query:
                short_query = query[:40] + "..." if len(query) > 40 else query
                return f"**WebSearch** `{short_query}`"
            return "**WebSearch** `query`"
        elif normalized_name == "Task":
            description = tool_input.get("description", "")
            subagent_type = tool_input.get("subagent_type", "")
            if description and subagent_type:
                return (
                    f"🤖 **Task** `{subagent_type}`\n> "
                    f"{description[:50]}{'...' if len(description) > 50 else ''}"
                )
            elif description:
                return f"🤖 **Task** `{description[:40]}{'...' if len(description) > 40 else ''}`"
            return "🤖 **Task** `subtask`"
        elif normalized_name == "ExitPlanMode":
            return "✅ **ExitPlanMode** `planning complete`"
        elif normalized_name == "NotebookEdit":
            notebook_path = tool_input.get("notebook_path", "")
            if notebook_path:
                filename = notebook_path.split("/")[-1]
                return f"📓 **NotebookEdit** `{filename}`"
            return "📓 **NotebookEdit** `notebook`"
        elif normalized_name == "MCPTool" or tool_name == "mcp_tool_call":
            server = tool_input.get("server", "")
            tool_name_inner = tool_input.get("tool", "")
            if server and tool_name_inner:
                return f"🔧 **MCP** `{server}.{tool_name_inner}`"
            return "🔧 **MCP** `tool call`"
        elif tool_name == "exec_command":
            command = tool_input.get("command", "")
            if command:
                display_cmd = command[:40] + "..." if len(command) > 40 else command
                return f"⚡ **Exec** `{display_cmd}`"
            return "⚡ **Exec** `command`"
        else:
            return f"**{tool_name}** `executing...`"



"""Qwen CLI provider implementation using ACP over stdio.

This adapter launches `qwen --experimental-acp`, speaks JSON-RPC over stdio,
and streams session/update notifications into our Message model. Thought
chunks are surfaced to the UI (unlike some providers that hide them).
"""
from __future__ import annotations

import asyncio
import base64
import json
import os
import uuid
from dataclasses import dataclass
import shutil
from datetime import datetime
from typing import Any, AsyncGenerator, Awaitable, Callable, Dict, List, Optional

from claudable_helper.core.terminal_ui import ui
from claudable_helper.models.messages import Message

from ..base import BaseCLI, CLIType, LineBuffer


@dataclass
class _Pending:
    fut: asyncio.Future


class _ACPClient:
    """Minimal JSON-RPC client over newline-delimited JSON on stdio."""

    def __init__(self, cmd: List[str], env: Optional[Dict[str, str]] = None, cwd: Optional[str] = None):
        self._cmd = cmd
        self._env = env or os.environ.copy()
        self._cwd = cwd or os.getcwd()
        self._proc: Optional[asyncio.subprocess.Process] = None
        self._next_id = 1
        self._pending: Dict[int, _Pending] = {}
        self._notif_handlers: Dict[str, List[Callable[[Dict[str, Any]], None]]] = {}
        self._request_handlers: Dict[str, Callable[[Dict[str, Any]], Awaitable[Dict[str, Any]]]] = {}
        self._reader_task: Optional[asyncio.Task] = None
        self._stderr_task: Optional[asyncio.Task] = None

    async def start(self) -> None:
        if self._proc is not None:
            return
        print(f"🔧 [Qwen] Starting ACP client with command: {self._cmd}")

        print(f"🔧 [Qwen] Current working directory: {self._cwd}")
        self._proc = await asyncio.create_subprocess_exec(
            *self._cmd,
            stdin=asyncio.subprocess.PIPE,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
            env=self._env,
            cwd=self._cwd,
        )

        # Start reader tasks
        self._reader_task = asyncio.create_task(self._reader_loop())
        self._stderr_task = asyncio.create_task(self._stderr_loop())

    async def stop(self) -> None:
        try:
            # Cancel pending futures first
            for pending in self._pending.values():
                if not pending.fut.done():
                    pending.fut.cancel()
            self._pending.clear()

            # Stop the process
            if self._proc and self._proc.returncode is None:
                self._proc.terminate()
                try:
                    await asyncio.wait_for(self._proc.wait(), timeout=2.0)
                except asyncio.TimeoutError:
                    self._proc.kill()
                    await self._proc.wait()
        finally:
            self._proc = None

            # Cancel reader tasks
            if self._reader_task:
                self._reader_task.cancel()
                try:
                    await self._reader_task
                except asyncio.CancelledError:
                    pass
                self._reader_task = None

            if self._stderr_task:
                self._stderr_task.cancel()
                try:
                    await self._stderr_task
                except asyncio.CancelledError:
                    pass
                self._stderr_task = None

    def on_notification(self, method: str, handler: Callable[[Dict[str, Any]], None]) -> None:
        self._notif_handlers.setdefault(method, []).append(handler)

    def off_notification(self, method: str, handler: Optional[Callable[[Dict[str, Any]], None]] = None) -> None:
        """Remove notification handler(s)."""
        if method not in self._notif_handlers:
            return

        if handler is None:
            # Remove all handlers for this method
            self._notif_handlers[method] = []
        else:
            # Remove specific handler
            self._notif_handlers[method] = [
                h for h in self._notif_handlers[method] if h != handler
            ]

    def on_request(self, method: str, handler: Callable[[Dict[str, Any]], Awaitable[Dict[str, Any]]]) -> None:
        self._request_handlers[method] = handler

    async def request(self, method: str, params: Optional[Dict[str, Any]] = None) -> Any:
        if not self._proc or not self._proc.stdin:
            raise RuntimeError("ACP process not started")
        msg_id = self._next_id
        self._next_id += 1
        fut: asyncio.Future = asyncio.get_running_loop().create_future()
        self._pending[msg_id] = _Pending(fut=fut)
        obj = {"jsonrpc": "2.0", "id": msg_id, "method": method, "params": params or {}}
        data = (json.dumps(obj) + "\n").encode("utf-8")
        self._proc.stdin.write(data)
        await self._proc.stdin.drain()
        return await fut

    async def _reader_loop(self) -> None:
        assert self._proc and self._proc.stdout
        stdout = self._proc.stdout
        reader = LineBuffer(stdout)
        buffer = b""
        while True:
            line = await reader.readline()
            if not line:
                break
            line = line.strip()
            if not line:
                continue
            try:
                msg = json.loads(line.decode("utf-8"))
            except Exception:
                # best-effort: ignore malformed
                continue

            # Response
            if isinstance(msg, dict) and "id" in msg and "method" not in msg:
                slot = self._pending.pop(int(msg["id"])) if int(msg["id"]) in self._pending else None
                if not slot:
                    continue
                if "error" in msg:
                    slot.fut.set_exception(RuntimeError(str(msg["error"])))
                else:
                    slot.fut.set_result(msg.get("result"))
                continue

            # Request from agent (client-side)
            if isinstance(msg, dict) and "method" in msg and "id" in msg:
                req_id = msg["id"]
                method = msg["method"]
                params = msg.get("params") or {}
                handler = self._request_handlers.get(method)
                if handler:
                    try:
                        result = await handler(params)
                        await self._send({"jsonrpc": "2.0", "id": req_id, "result": result})
                    except Exception as e:
                        await self._send({
                            "jsonrpc": "2.0",
                            "id": req_id,
                            "error": {"code": -32000, "message": str(e)},
                        })
                else:
                    await self._send({
                        "jsonrpc": "2.0",
                        "id": req_id,
                        "error": {"code": -32601, "message": "Method not found"},
                    })
                continue

            # Notification from agent
            if isinstance(msg, dict) and "method" in msg and "id" not in msg:
                method = msg["method"]
                params = msg.get("params") or {}
                for h in self._notif_handlers.get(method, []) or []:
                    try:
                        h(params)
                    except Exception:
                        pass

    async def _send(self, obj: Dict[str, Any]) -> None:
        if not self._proc or not self._proc.stdin:
            return
        self._proc.stdin.write((json.dumps(obj) + "\n").encode("utf-8"))
        await self._proc.stdin.drain()

    async def _stderr_loop(self) -> None:
        """Background task to drain stderr to prevent blocking."""
        if not self._proc or not self._proc.stderr:
            return

        stderr = self._proc.stderr
        stderr_reader = LineBuffer(stderr)
        try:
            while True:
                line = await stderr_reader.readline()
                if not line:
                    break
                # Optionally log or process stderr
                # For now, just drain it silently
        except asyncio.CancelledError:
            pass
        except Exception:
            pass


class QwenCLI(BaseCLI):
    """Qwen CLI via ACP. Streams message and thought chunks to UI."""

    # Shared ACP client across instances to preserve sessions
    _SHARED_CLIENT: Optional[_ACPClient] = None
    _SHARED_INITIALIZED: bool = False

    def __init__(self, db_session=None):
        super().__init__(CLIType.QWEN)
        self.db_session = db_session
        self._session_store: Dict[str, str] = {}
        self._client: Optional[_ACPClient] = None
        self._initialized = False

    async def check_availability(self) -> Dict[str, Any]:
        try:
            proc = await asyncio.create_subprocess_shell(
                "qwen --help",
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
            )
            stdout, stderr = await proc.communicate()
            if proc.returncode != 0:
                return {
                    "available": False,
                    "configured": False,
                    "error": "Qwen CLI not found. Install Qwen CLI and ensure it is in PATH.",
                }
            return {
                "available": True,
                "configured": True,
                "models": self.get_supported_models(),
                "default_models": [],
            }
        except Exception as e:
            return {"available": False, "configured": False, "error": str(e)}

    async def _ensure_provider_md(self, project_path: str) -> None:
        """Ensure QWEN.md exists at the project repo root.

        Mirrors CursorAgent behavior: copy app/prompt/system-prompt.md if present.
        """
        try:
            project_repo_path = os.path.join(project_path, "repo")
            if not os.path.exists(project_repo_path):
                project_repo_path = project_path
            md_path = os.path.join(project_repo_path, "QWEN.md")
            if os.path.exists(md_path):
                ui.debug(f"QWEN.md already exists at: {md_path}", "Qwen")
                return
            current_file_dir = os.path.dirname(os.path.abspath(__file__))
            app_dir = os.path.abspath(os.path.join(current_file_dir, "..", "..", ".."))
            system_prompt_path = os.path.join(app_dir, "prompt", "system-prompt.md")
            content = "# QWEN\n\n"
            if os.path.exists(system_prompt_path):
                try:
                    with open(system_prompt_path, "r", encoding="utf-8") as f:
                        content += f.read()
                except Exception:
                    pass
            with open(md_path, "w", encoding="utf-8") as f:
                f.write(content)
            ui.success(f"Created QWEN.md at: {md_path}", "Qwen")
        except Exception as e:
            ui.warning(f"Failed to create QWEN.md: {e}", "Qwen")

    async def _ensure_client(self) -> _ACPClient:
        # Use shared client across adapter instances
        if QwenCLI._SHARED_CLIENT is None:
            # Resolve command: env(QWEN_CMD) -> qwen -> qwen-code
            candidates = []
            env_cmd = os.getenv("QWEN_CMD")
            if env_cmd:
                candidates.append(env_cmd)
            candidates.extend(["qwen", "qwen-code"])
            resolved = None
            for c in candidates:
                if shutil.which(c):
                    resolved = c
                    break
            if not resolved:
                raise RuntimeError(
                    "Qwen CLI not found. Set QWEN_CMD or install 'qwen' CLI in PATH."
                )
            cmd = [resolved, "--experimental-acp"]
            # Prefer device-code / no-browser flow to avoid launching windows
            env = os.environ.copy()
            env.setdefault("NO_BROWSER", "1")
            QwenCLI._SHARED_CLIENT = _ACPClient(cmd, env=env)

            # Register client-side request handlers
            async def _handle_permission(params: Dict[str, Any]) -> Dict[str, Any]:
                # Auto-approve: prefer allow_always -> allow_once -> first
                options = params.get("options") or []
                chosen = None
                for kind in ("allow_always", "allow_once"):
                    chosen = next((o for o in options if o.get("kind") == kind), None)
                    if chosen:
                        break
                if not chosen and options:
                    chosen = options[0]
                if not chosen:
                    return {"outcome": {"outcome": "cancelled"}}
                return {
                    "outcome": {"outcome": "selected", "optionId": chosen.get("optionId")}
                }

            async def _fs_read(params: Dict[str, Any]) -> Dict[str, Any]:
                # Conservative: deny reading arbitrary files from agent perspective
                return {"content": ""}

            async def _fs_write(params: Dict[str, Any]) -> Dict[str, Any]:
                # Validate required parameters for file editing
                if "old_string" not in params and "content" in params:
                    # If old_string is missing but content exists, log warning
                    ui.warning(
                        f"Qwen edit missing 'old_string' parameter: {params.get('path', 'unknown')}",
                        "Qwen"
                    )
                    return {"error": "Missing required parameter: old_string"}
                # Not fully implemented for safety, but return success to avoid blocking
                return {"success": True}

            async def _edit_file(params: Dict[str, Any]) -> Dict[str, Any]:
                # Handle edit requests with proper parameter validation
                path = params.get('path', params.get('file_path', 'unknown'))
                
                # Log the edit attempt for debugging
                ui.debug(f"Qwen edit request: path={path}, has_old_string={'old_string' in params}", "Qwen")
                
                if "old_string" not in params:
                    ui.warning(
                        f"Qwen edit missing 'old_string': {path}",
                        "Qwen"
                    )
                    # Return success anyway to not block Qwen's workflow
                    # This allows Qwen to continue even with malformed requests
                    return {"success": True}
                
                # For safety, we don't actually perform the edit but return success
                ui.debug(f"Qwen edit would modify: {path}", "Qwen")
                return {"success": True}

            QwenCLI._SHARED_CLIENT.on_request("session/request_permission", _handle_permission)
            QwenCLI._SHARED_CLIENT.on_request("fs/read_text_file", _fs_read)
            QwenCLI._SHARED_CLIENT.on_request("fs/write_text_file", _fs_write)
            QwenCLI._SHARED_CLIENT.on_request("edit", _edit_file)
            QwenCLI._SHARED_CLIENT.on_request("str_replace_editor", _edit_file)

            await QwenCLI._SHARED_CLIENT.start()
            # Attach simple stderr logger (filtering out polling messages)
            try:
                proc = QwenCLI._SHARED_CLIENT._proc
                if proc and proc.stderr:
                    async def _log_stderr(stream):
                        stderr_reader = LineBuffer(stream)
                        while True:
                            line = await stderr_reader.readline()
                            if not line:
                                break
                            decoded = line.decode(errors="ignore").strip()
                            # Skip polling for token messages
                            if "polling for token" in decoded.lower():
                                continue
                            # Skip ImportProcessor errors (these are just warnings about npm packages)
                            if "[ERROR] [ImportProcessor]" in decoded:
                                continue
                            # Skip ENOENT errors for node_modules paths
                            if "ENOENT" in decoded and ("node_modules" in decoded or "tailwind" in decoded or "supabase" in decoded):
                                continue
                            # Only log meaningful errors
                            if decoded and not decoded.startswith("DEBUG"):
                                ui.warning(decoded, "Qwen STDERR")
                    asyncio.create_task(_log_stderr(proc.stderr))
            except Exception:
                pass

        self._client = QwenCLI._SHARED_CLIENT

        if not QwenCLI._SHARED_INITIALIZED:
            try:
                await self._client.request(
                    "initialize",
                    {
                        "clientCapabilities": {
                            "fs": {"readTextFile": False, "writeTextFile": False}
                        },
                        "protocolVersion": 1,
                    },
                )
                QwenCLI._SHARED_INITIALIZED = True
            except Exception as e:
                ui.error(f"Qwen initialize failed: {e}", "Qwen")
                raise

        return self._client

    async def execute_with_streaming(
        self,
        instruction: str,
        project_path: str,
        session_id: Optional[str] = None,
        log_callback: Optional[Callable[[str], Any]] = None,
        images: Optional[List[Dict[str, Any]]] = None,
        model: Optional[str] = None,
        is_initial_prompt: bool = False,
    ) -> AsyncGenerator[Message, None]:
        client = await self._ensure_client()
        # Ensure provider markdown exists in project repo
        await self._ensure_provider_md(project_path)
        turn_id = str(uuid.uuid4())[:8]
        try:
            ui.debug(
                f"[{turn_id}] execute_with_streaming start | model={model or '-'} | images={len(images or [])} | instruction_len={len(instruction or '')}",
                "Qwen",
            )
        except Exception:
            pass

        # Resolve repo cwd
        project_repo_path = os.path.join(project_path, "repo")
        if not os.path.exists(project_repo_path):
            project_repo_path = project_path

        # Project ID
        path_parts = project_path.split("/")
        project_id = (
            path_parts[path_parts.index("repo") - 1]
            if "repo" in path_parts and path_parts.index("repo") > 0
            else path_parts[-1]
        )

        # Ensure session
        stored_session_id = await self.get_session_id(project_id)
        if not stored_session_id:
            # Try to reuse cached OAuth by creating a session first
            try:
                result = await client.request(
                    "session/new", {"cwd": project_repo_path, "mcpServers": []}
                )
                stored_session_id = result.get("sessionId")
                if stored_session_id:
                    await self.set_session_id(project_id, stored_session_id)
                    ui.info(f"Qwen session created: {stored_session_id}", "Qwen")
            except Exception as e:
                # Authenticate only if needed, then retry session/new
                auth_method = os.getenv("QWEN_AUTH_METHOD", "qwen-oauth")
                ui.warning(
                    f"Qwen session/new failed; authenticating via {auth_method}: {e}",
                    "Qwen",
                )
                try:
                    await client.request("authenticate", {"methodId": auth_method})
                    result = await client.request(
                        "session/new", {"cwd": project_repo_path, "mcpServers": []}
                    )
                    stored_session_id = result.get("sessionId")
                    if stored_session_id:
                        await self.set_session_id(project_id, stored_session_id)
                        ui.info(
                            f"Qwen session created after auth: {stored_session_id}", "Qwen"
                        )
                except Exception as e2:
                    err = f"Qwen authentication/session failed: {e2}"
                    yield Message(
                        id=str(uuid.uuid4()),
                        project_id=project_path,
                        role="assistant",
                        message_type="error",
                        content=err,
                        metadata_json={"cli_type": self.cli_type.value},
                        session_id=session_id,
                        created_at=datetime.utcnow(),
                    )
                    return

        # Subscribe to session/update notifications and stream as Message
        q: asyncio.Queue = asyncio.Queue()
        thought_buffer: List[str] = []
        text_buffer: List[str] = []

        def _on_update(params: Dict[str, Any]) -> None:
            try:
                if params.get("sessionId") != stored_session_id:
                    return
                update = params.get("update") or {}
                q.put_nowait(update)
            except Exception:
                pass

        client.on_notification("session/update", _on_update)

        # Build prompt parts
        parts: List[Dict[str, Any]] = []
        if instruction:
            parts.append({"type": "text", "text": instruction})

        # Qwen Coder currently does not support image input.
        # If images are provided, ignore them to avoid ACP errors.
        if images:
            try:
                ui.warning(
                    "Qwen Coder does not support image input yet. Ignoring attached images.",
                    "Qwen",
                )
            except Exception:
                pass

        # Send prompt request
        # Helper to create a prompt task for current session
        def _make_prompt_task() -> asyncio.Task:
            ui.debug(f"[{turn_id}] sending session/prompt (parts={len(parts)})", "Qwen")
            return asyncio.create_task(
                client.request(
                    "session/prompt",
                    {"sessionId": stored_session_id, "prompt": parts},
                )
            )

        prompt_task = _make_prompt_task()

        # Stream notifications until prompt completes
        while True:
            done, pending = await asyncio.wait(
                {prompt_task, asyncio.create_task(q.get())},
                return_when=asyncio.FIRST_COMPLETED,
            )
            if prompt_task in done:
                ui.debug(f"[{turn_id}] prompt_task completed; draining updates", "Qwen")
                # Flush remaining updates quickly
                while not q.empty():
                    update = q.get_nowait()
                    async for m in self._update_to_messages(update, project_path, session_id, thought_buffer, text_buffer):
                        if m:
                            yield m
                # Handle prompt exception (e.g., session not found) with one retry
                exc = prompt_task.exception()
                if exc:
                    msg = str(exc)
                    if "Session not found" in msg or "session not found" in msg.lower():
                        ui.warning("Qwen session expired; creating a new session and retrying", "Qwen")
                        try:
                            result = await client.request(
                                "session/new", {"cwd": project_repo_path, "mcpServers": []}
                            )
                            stored_session_id = result.get("sessionId")
                            if stored_session_id:
                                await self.set_session_id(project_id, stored_session_id)
                                prompt_task = _make_prompt_task()
                                continue  # re-enter wait loop
                        except Exception as e2:
                            yield Message(
                                id=str(uuid.uuid4()),
                                project_id=project_path,
                                role="assistant",
                                message_type="error",
                                content=f"Qwen session recovery failed: {e2}",
                                metadata_json={"cli_type": self.cli_type.value},
                                session_id=session_id,
                                created_at=datetime.utcnow(),
                            )
                    else:
                        yield Message(
                            id=str(uuid.uuid4()),
                            project_id=project_path,
                            role="assistant",
                            message_type="error",
                            content=f"Qwen prompt error: {msg}",
                            metadata_json={"cli_type": self.cli_type.value},
                            session_id=session_id,
                            created_at=datetime.utcnow(),
                        )
                # Final flush of buffered assistant text
                if thought_buffer or text_buffer:
                    yield Message(
                        id=str(uuid.uuid4()),
                        project_id=project_path,
                        role="assistant",
                        message_type="chat",
                        content=self._compose_content(thought_buffer, text_buffer),
                        metadata_json={"cli_type": self.cli_type.value},
                        session_id=session_id,
                        created_at=datetime.utcnow(),
                    )
                    thought_buffer.clear()
                    text_buffer.clear()
                break

            # Process one update
            for task in done:
                if task is not prompt_task:
                    update = task.result()
                    # Suppress verbose per-chunk logs; log only tool calls below
                    async for m in self._update_to_messages(update, project_path, session_id, thought_buffer, text_buffer):
                        if m:
                            yield m

        # Yield hidden result/system message for bookkeeping
        yield Message(
            id=str(uuid.uuid4()),
            project_id=project_path,
            role="system",
            message_type="result",
            content="Qwen turn completed",
            metadata_json={"cli_type": self.cli_type.value, "hidden_from_ui": True},
            session_id=session_id,
            created_at=datetime.utcnow(),
        )
        ui.info(f"[{turn_id}] turn completed", "Qwen")

    async def _update_to_messages(
        self,
        update: Dict[str, Any],
        project_path: str,
        session_id: Optional[str],
        thought_buffer: List[str],
        text_buffer: List[str],
    ) -> AsyncGenerator[Optional[Message], None]:
        kind = update.get("sessionUpdate") or update.get("type")
        now = datetime.utcnow()
        if kind in ("agent_message_chunk", "agent_thought_chunk"):
            text = ((update.get("content") or {}).get("text")) or update.get("text") or ""
            if not isinstance(text, str):
                text = str(text)
            if kind == "agent_thought_chunk":
                thought_buffer.append(text)
            else:
                text_buffer.append(text)
            # Do not flush here: we flush only before tool events or at end,
            # to match result_qwen.md behavior (message → tools → message ...)
            return
        elif kind in ("tool_call", "tool_call_update"):
            # Qwen emits frequent tool_call_update events and opaque call IDs
            # like `call_390e...` that produce noisy "executing..." lines.
            # Hide updates entirely and only surface meaningful tool calls.
            if kind == "tool_call_update":
                return

            tool_name = self._parse_tool_name(update)
            tool_input = self._extract_tool_input(update)
            summary = self._create_tool_summary(tool_name, tool_input)

            # Suppress unknown/opaque tool names that fall back to "executing..."
            try:
                tn = (tool_name or "").lower()
                is_opaque = (
                    tn in ("call", "tool", "toolcall")
                    or tn.startswith("call_")
                    or tn.startswith("call-")
                )
                if is_opaque or summary.strip().endswith("`executing...`"):
                    return
            except Exception:
                pass

            # Flush chat buffer before showing tool usage
            if thought_buffer or text_buffer:
                yield Message(
                    id=str(uuid.uuid4()),
                    project_id=project_path,
                    role="assistant",
                    message_type="chat",
                    content=self._compose_content(thought_buffer, text_buffer),
                    metadata_json={"cli_type": self.cli_type.value},
                    session_id=session_id,
                    created_at=now,
                )
                thought_buffer.clear()
                text_buffer.clear()

            # Show tool use as a visible message
            yield Message(
                id=str(uuid.uuid4()),
                project_id=project_path,
                role="assistant",
                message_type="tool_use",
                content=summary,
                metadata_json={
                    "cli_type": self.cli_type.value,
                    "event_type": "tool_call",  # normalized
                    "tool_name": tool_name,
                    "tool_input": tool_input,
                },
                session_id=session_id,
                created_at=now,
            )
            # Concise server-side log
            try:
                path = tool_input.get("path")
                ui.info(
                    f"TOOL {tool_name.upper()}" + (f" {path}" if path else ""),
                    "Qwen",
                )
            except Exception:
                pass
        elif kind == "plan":
            entries = update.get("entries") or []
            lines = []
            for e in entries[:6]:
                title = e.get("title") if isinstance(e, dict) else str(e)
                if title:
                    lines.append(f"• {title}")
            content = "\n".join(lines) if lines else "Planning…"
            # Optionally flush buffer before plan (keep as separate status)
            if thought_buffer or text_buffer:
                yield Message(
                    id=str(uuid.uuid4()),
                    project_id=project_path,
                    role="assistant",
                    message_type="chat",
                    content=self._compose_content(thought_buffer, text_buffer),
                    metadata_json={"cli_type": self.cli_type.value},
                    session_id=session_id,
                    created_at=now,
                )
                thought_buffer.clear()
                text_buffer.clear()
            yield Message(
                id=str(uuid.uuid4()),
                project_id=project_path,
                role="assistant",
                message_type="chat",
                content=content,
                metadata_json={"cli_type": self.cli_type.value, "event_type": "plan"},
                session_id=session_id,
                created_at=now,
            )
        else:
            # Unknown update kinds ignored
            return

    def _compose_content(self, thought_buffer: List[str], text_buffer: List[str]) -> str:
        # Qwen formatting per result_qwen.md: merge thoughts + text, and filter noisy call_* lines
        import re
        parts: List[str] = []
        if thought_buffer:
            parts.append("".join(thought_buffer))
            if text_buffer:
                parts.append("\n\n")
        if text_buffer:
            parts.append("".join(text_buffer))
        combined = "".join(parts)
        # Remove lines like: call_XXXXXXXX executing... (Qwen internal call IDs)
        combined = re.sub(r"(?m)^call[_-][A-Za-z0-9]+.*$\n?", "", combined)
        # Trim excessive blank lines
        combined = re.sub(r"\n{3,}", "\n\n", combined).strip()
        return combined

    def _parse_tool_name(self, update: Dict[str, Any]) -> str:
        # Prefer explicit kind from Qwen events
        kind = update.get("kind")
        if isinstance(kind, str) and kind.strip():
            return kind.strip()
        # Fallback: derive from toolCallId by splitting on '-' or '_'
        raw_id = update.get("toolCallId") or ""
        if isinstance(raw_id, str) and raw_id:
            for sep in ("-", "_"):
                base = raw_id.split(sep, 1)[0]
                if base and base.lower() not in ("call", "tool", "toolcall"):
                    return base
        return update.get("title") or "tool"

    def _extract_tool_input(self, update: Dict[str, Any]) -> Dict[str, Any]:
        tool_input: Dict[str, Any] = {}
        path: Optional[str] = None
        locs = update.get("locations")
        if isinstance(locs, list) and locs:
            first = locs[0]
            if isinstance(first, dict):
                path = (
                    first.get("path")
                    or first.get("file")
                    or first.get("file_path")
                    or first.get("filePath")
                    or first.get("uri")
                )
                if isinstance(path, str) and path.startswith("file://"):
                    path = path[len("file://"):]
        if not path:
            content = update.get("content")
            if isinstance(content, list):
                for c in content:
                    if isinstance(c, dict):
                        cand = (
                            c.get("path")
                            or c.get("file")
                            or c.get("file_path")
                            or (c.get("args") or {}).get("path")
                        )
                        if cand:
                            path = cand
                            break
        if path:
            tool_input["path"] = str(path)
        return tool_input

    async def get_session_id(self, project_id: str) -> Optional[str]:
        if self.db_session:
            try:
                from claudable_helper.models.projects import Project

                project = (
                    self.db_session.query(Project)
                    .filter(Project.id == project_id)
                    .first()
                )
                if project and project.active_cursor_session_id:
                    try:
                        data = json.loads(project.active_cursor_session_id)
                        if isinstance(data, dict) and "qwen" in data:
                            return data["qwen"]
                    except Exception:
                        pass
            except Exception as e:
                ui.warning(f"Qwen get_session_id DB error: {e}", "Qwen")
        return self._session_store.get(project_id)

    async def set_session_id(self, project_id: str, session_id: str) -> None:
        if self.db_session:
            try:
                from claudable_helper.models.projects import Project

                project = (
                    self.db_session.query(Project)
                    .filter(Project.id == project_id)
                    .first()
                )
                if project:
                    data: Dict[str, Any] = {}
                    if project.active_cursor_session_id:
                        try:
                            val = json.loads(project.active_cursor_session_id)
                            if isinstance(val, dict):
                                data = val
                            else:
                                data = {"cursor": val}
                        except Exception:
                            data = {"cursor": project.active_cursor_session_id}
                    data["qwen"] = session_id
                    project.active_cursor_session_id = json.dumps(data)
                    self.db_session.commit()
            except Exception as e:
                ui.warning(f"Qwen set_session_id DB error: {e}", "Qwen")
        self._session_store[project_id] = session_id


def _mime_for(path: str) -> str:
    p = path.lower()
    if p.endswith(".png"):
        return "image/png"
    if p.endswith(".jpg") or p.endswith(".jpeg"):
        return "image/jpeg"
    if p.endswith(".gif"):
        return "image/gif"
    if p.endswith(".webp"):
        return "image/webp"
    if p.endswith(".bmp"):
        return "image/bmp"
    return "application/octet-stream"


__all__ = ["QwenCLI"]



"""Mock message models for claudable_helper.

This module provides mock implementations of the message models
that were originally imported from app.models.messages.
"""
import uuid
from datetime import datetime
from enum import Enum
from typing import Any, Dict, List, Optional


class MessageType(str, Enum):
    """Message type enumeration."""
    USER = "user"
    ASSISTANT = "assistant"
    SYSTEM = "system"
    TOOL = "tool"
    ERROR = "error"
    CHAT = "chat"
    TOOL_USE = "tool_use"
    TOOL_RESULT = "tool_result"
    RESULT = "result"


class MessageStatus(str, Enum):
    """Message status enumeration."""
    PENDING = "pending"
    STREAMING = "streaming"
    COMPLETED = "completed"
    FAILED = "failed"


class Message:
    """Mock message model for chat interactions."""
    
    def __init__(
        self,
        content: str = "",
        message_type: MessageType = MessageType.USER,
        session_id: Optional[str] = None,
        project_id: Optional[str] = None,
        model: Optional[str] = None,
        status: MessageStatus = MessageStatus.COMPLETED,
        metadata: Optional[Dict[str, Any]] = None,
        message_id: Optional[str] = None,
        created_at: Optional[datetime] = None,
        updated_at: Optional[datetime] = None,
        # Additional parameters for compatibility with CLI adapters
        id: Optional[str] = None,
        role: Optional[str] = None,
        metadata_json: Optional[Dict[str, Any]] = None,
    ):
        # Handle id parameter (CLI adapters pass 'id', we prefer 'message_id')
        self.id = id or message_id or str(uuid.uuid4())
        self.content = content

        # Handle message_type as either string or enum
        if isinstance(message_type, str):
            self.message_type = MessageType(message_type)
        else:
            self.message_type = message_type

        self.session_id = session_id
        self.project_id = project_id
        self.model = model

        # Handle metadata (merge metadata_json if provided)
        self.metadata = metadata or {}
        if metadata_json:
            self.metadata.update(metadata_json)

        # Handle status as either string or enum
        if isinstance(status, str):
            self.status = MessageStatus(status)
        else:
            self.status = status

        self.metadata = metadata or {}
        self.created_at = created_at or datetime.utcnow()
        self.updated_at = updated_at or datetime.utcnow()

        # Store role - use provided role or fallback to message_type value
        self.role = role if role is not None else self.message_type.value
        self.timestamp = self.created_at
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert message to dictionary."""
        return {
            "id": self.id,
            "content": self.content,
            "message_type": self.message_type.value,
            "role": self.role,
            "session_id": self.session_id,
            "project_id": self.project_id,
            "model": self.model,
            "status": self.status.value,
            "metadata": self.metadata,
            "created_at": self.created_at.isoformat() if self.created_at else None,
            "updated_at": self.updated_at.isoformat() if self.updated_at else None,
        }
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Message":
        """Create message from dictionary."""
        return cls(
            message_id=data.get("id"),
            content=data.get("content", ""),
            message_type=MessageType(data.get("message_type", "user")),
            session_id=data.get("session_id"),
            project_id=data.get("project_id"),
            model=data.get("model"),
            status=MessageStatus(data.get("status", "completed")),
            metadata=data.get("metadata", {}),
            created_at=datetime.fromisoformat(data["created_at"]) if data.get("created_at") else None,
            updated_at=datetime.fromisoformat(data["updated_at"]) if data.get("updated_at") else None,
        )
    
    def update_content(self, content: str) -> None:
        """Update message content."""
        self.content = content
        self.updated_at = datetime.utcnow()
    
    def update_status(self, status: MessageStatus) -> None:
        """Update message status."""
        self.status = status
        self.updated_at = datetime.utcnow()
    
    def __str__(self) -> str:
        """String representation of the message."""
        return f"Message(id={self.id}, type={self.message_type.value}, content='{self.content[:50]}...')"
    
    def __repr__(self) -> str:
        """Detailed string representation."""
        return (
            f"Message(id={self.id}, type={self.message_type.value}, "
            f"status={self.status.value}, session_id={self.session_id})"
        )


import os
from pathlib import Path
from typing import Optional


def ensure_dir(path: str) -> None:
    Path(path).mkdir(parents=True, exist_ok=True)


def write_bytes(path: str, data: bytes) -> None:
    ensure_dir(str(Path(path).parent))
    with open(path, "wb") as f:
        f.write(data)


def write_text(path: str, data: str) -> None:
    ensure_dir(str(Path(path).parent))
    with open(path, "w", encoding="utf-8") as f:
        f.write(data)



from .claude_code import ClaudeCodeCLI
from .cursor_agent import CursorAgentCLI
from .codex_cli import CodexCLI
from .qwen_cli import QwenCLI
from .gemini_cli import GeminiCLI

__all__ = [
    "ClaudeCodeCLI",
    "CursorAgentCLI",
    "CodexCLI",
    "QwenCLI",
    "GeminiCLI",
]



"""Mock environment variable models for claudable_helper.

This module provides mock implementations of the environment variable models
that were originally imported from app.models.env_vars.
"""
import os
import uuid
from datetime import datetime
from enum import Enum
from typing import Any, Dict, Optional


class EnvVarType(str, Enum):
    """Environment variable type enumeration."""
    STRING = "string"
    NUMBER = "number"
    BOOLEAN = "boolean"
    SECRET = "secret"
    JSON = "json"


class EnvVarScope(str, Enum):
    """Environment variable scope enumeration."""
    GLOBAL = "global"
    PROJECT = "project"
    SESSION = "session"


class EnvVar:
    """Mock environment variable model for managing configuration."""
    
    def __init__(
        self,
        key: str,
        value: str,
        var_type: EnvVarType = EnvVarType.STRING,
        scope: EnvVarScope = EnvVarScope.PROJECT,
        project_id: Optional[str] = None,
        env_id: Optional[str] = None,
        description: Optional[str] = None,
        is_encrypted: bool = False,
        metadata: Optional[Dict[str, Any]] = None,
        created_at: Optional[datetime] = None,
        updated_at: Optional[datetime] = None,
    ):
        self.id = env_id or str(uuid.uuid4())
        self.key = key
        self.value = value
        self.var_type = var_type
        self.scope = scope
        self.project_id = project_id
        self.description = description
        self.is_encrypted = is_encrypted
        self.metadata = metadata or {}
        self.created_at = created_at or datetime.utcnow()
        self.updated_at = updated_at or datetime.utcnow()
    
    @property
    def is_secret(self) -> bool:
        """Check if this is a secret environment variable."""
        return self.var_type == EnvVarType.SECRET or self.is_encrypted
    
    @property
    def masked_value(self) -> str:
        """Get masked version of the value for display."""
        if not self.is_secret:
            return self.value
        
        if len(self.value) <= 8:
            return "*" * len(self.value)
        
        return self.value[:2] + "*" * (len(self.value) - 4) + self.value[-2:]
    
    @property
    def display_value(self) -> str:
        """Get appropriate value for display (masked if secret)."""
        return self.masked_value if self.is_secret else self.value
    
    def get_typed_value(self) -> Any:
        """Get value converted to the appropriate type."""
        if self.var_type == EnvVarType.BOOLEAN:
            return self.value.lower() in ("true", "1", "yes", "on")
        elif self.var_type == EnvVarType.NUMBER:
            try:
                if "." in self.value:
                    return float(self.value)
                return int(self.value)
            except ValueError:
                return self.value
        elif self.var_type == EnvVarType.JSON:
            try:
                import json
                return json.loads(self.value)
            except (json.JSONDecodeError, ImportError):
                return self.value
        else:
            return self.value
    
    def set_value(self, value: str) -> None:
        """Set new value for the environment variable."""
        self.value = value
        self.updated_at = datetime.utcnow()
    
    def apply_to_environment(self) -> None:
        """Apply this environment variable to the current process environment."""
        os.environ[self.key] = self.value
    
    @classmethod
    def from_environment(cls, key: str, **kwargs) -> Optional["EnvVar"]:
        """Create EnvVar from current environment."""
        value = os.environ.get(key)
        if value is None:
            return None
        
        return cls(key=key, value=value, **kwargs)
    
    def to_dict(self, include_value: bool = False, mask_secrets: bool = True) -> Dict[str, Any]:
        """Convert environment variable to dictionary.
        
        Args:
            include_value: Whether to include the actual value
            mask_secrets: Whether to mask secret values
        """
        data = {
            "id": self.id,
            "key": self.key,
            "var_type": self.var_type.value,
            "scope": self.scope.value,
            "project_id": self.project_id,
            "description": self.description,
            "is_encrypted": self.is_encrypted,
            "is_secret": self.is_secret,
            "metadata": self.metadata,
            "created_at": self.created_at.isoformat() if self.created_at else None,
            "updated_at": self.updated_at.isoformat() if self.updated_at else None,
        }
        
        if include_value:
            if mask_secrets and self.is_secret:
                data["value"] = self.masked_value
            else:
                data["value"] = self.value
        
        return data
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "EnvVar":
        """Create environment variable from dictionary."""
        return cls(
            env_id=data.get("id"),
            key=data.get("key", ""),
            value=data.get("value", ""),
            var_type=EnvVarType(data.get("var_type", "string")),
            scope=EnvVarScope(data.get("scope", "project")),
            project_id=data.get("project_id"),
            description=data.get("description"),
            is_encrypted=data.get("is_encrypted", False),
            metadata=data.get("metadata", {}),
            created_at=datetime.fromisoformat(data["created_at"]) if data.get("created_at") else None,
            updated_at=datetime.fromisoformat(data["updated_at"]) if data.get("updated_at") else None,
        )
    
    def __str__(self) -> str:
        """String representation of the environment variable."""
        return f"EnvVar(key={self.key}, type={self.var_type.value})"
    
    def __repr__(self) -> str:
        """Detailed string representation."""
        return (
            f"EnvVar(id={self.id}, key={self.key}, type={self.var_type.value}, "
            f"scope={self.scope.value}, is_secret={self.is_secret})"
        )


"""
Project Initializer Service
Handles project initialization, scaffolding, and setup
"""
import os
import json
import shutil
from pathlib import Path
from typing import Optional

from claudable_helper.core.config import settings
# Commenting out filesystem import - this needs to be updated or mocked
# from app.services.filesystem import (
    ensure_dir,
    scaffold_nextjs_minimal,
    init_git_repo,
    write_env_file
)


async def initialize_project(project_id: str, name: str) -> str:
    """
    Initialize a new project with directory structure and scaffolding
    
    Args:
        project_id: Unique project identifier
        name: Human-readable project name
    
    Returns:
        str: Path to the created project directory
    """
    
    # Create project directory
    project_path = os.path.join(settings.projects_root, project_id, "repo")
    ensure_dir(project_path)
    
    # Create assets directory
    assets_path = os.path.join(settings.projects_root, project_id, "assets")
    ensure_dir(assets_path)
    
    try:
        # Scaffold NextJS project using create-next-app (includes automatic git init)
        scaffold_nextjs_minimal(project_path)
        
        # CRITICAL: Force create independent git repository for each project
        # create-next-app inherits parent .git when run inside existing repo
        # This ensures each project has its own isolated git history
        init_git_repo(project_path)
        
        # Create initial .env file
        env_content = f"NEXT_PUBLIC_PROJECT_ID={project_id}\nNEXT_PUBLIC_PROJECT_NAME={name}\n"
        write_env_file(project_path, env_content)
        
        # Create metadata directory and initial metadata file
        create_project_metadata(project_id, name)
        
        # Setup Claude Code configuration
        setup_claude_config(project_path)
        
        return project_path
        
    except Exception as e:
        # Clean up failed project directory
        import shutil
        project_root = os.path.join(settings.projects_root, project_id)
        if os.path.exists(project_root):
            shutil.rmtree(project_root)
        
        # Re-raise with user-friendly message
        raise Exception(f"Failed to initialize Next.js project: {str(e)}")


async def cleanup_project(project_id: str) -> bool:
    """
    Clean up project files and directories. Be robust against running preview
    processes, transient filesystem locks, and read-only files.

    Args:
        project_id: Project identifier to clean up

    Returns:
        bool: True if cleanup was successful
    """

    project_root = os.path.join(settings.projects_root, project_id)

    # Nothing to do
    if not os.path.exists(project_root):
        return False

    # 1) Ensure any running preview processes for this project are terminated
    try:
        # Mock cleanup function
        def cleanup_project_resources(*args, **kwargs):
            pass  # Placeholder for cleanup logic
        cleanup_project_resources(project_id)
    except Exception as e:
        # Do not fail cleanup because of process stop errors
        print(f"[cleanup] Warning: failed stopping preview process for {project_id}: {e}")

    # 2) Robust recursive deletion with retries
    import time
    import errno
    import stat
    import shutil

    def _onerror(func, path, exc_info):
        # Try to chmod and retry if permission error
        try:
            if not os.path.exists(path):
                return
            os.chmod(path, stat.S_IWUSR | stat.S_IRUSR | stat.S_IXUSR)
            func(path)
        except Exception:
            pass

    attempts = 0
    max_attempts = 5
    last_err = None
    while attempts < max_attempts:
        try:
            shutil.rmtree(project_root, onerror=_onerror)
            return True
        except OSError as e:
            last_err = e
            # On macOS, ENOTEMPTY (66) or EBUSY can happen if watchers are active
            if e.errno in (errno.ENOTEMPTY, errno.EBUSY, 66):
                time.sleep(0.25 * (attempts + 1))
                attempts += 1
                continue
            else:
                print(f"Error cleaning up project {project_id}: {e}")
                return False
        except Exception as e:
            last_err = e
            print(f"Error cleaning up project {project_id}: {e}")
            return False

    # Final attempt to handle lingering dotfiles
    try:
        # Remove remaining leaf entries then rmdir tree if any
        for root, dirs, files in os.walk(project_root, topdown=False):
            for name in files:
                try:
                    os.remove(os.path.join(root, name))
                except Exception:
                    pass
            for name in dirs:
                try:
                    os.rmdir(os.path.join(root, name))
                except Exception:
                    pass
        os.rmdir(project_root)
        return True
    except Exception as e:
        print(f"Error cleaning up project {project_id}: {e if e else last_err}")
        return False


async def get_project_path(project_id: str) -> Optional[str]:
    """
    Get the filesystem path for a project
    
    Args:
        project_id: Project identifier
    
    Returns:
        Optional[str]: Path to project directory if it exists
    """
    
    project_path = os.path.join(settings.projects_root, project_id, "repo")
    
    if os.path.exists(project_path):
        return project_path
    
    return None


async def project_exists(project_id: str) -> bool:
    """
    Check if a project exists on the filesystem
    
    Args:
        project_id: Project identifier
    
    Returns:
        bool: True if project exists
    """
    
    project_path = os.path.join(settings.projects_root, project_id)
    return os.path.exists(project_path)


def create_project_metadata(project_id: str, name: str):
    """
    Create initial metadata file with placeholder content
    This will be filled by CLI Agent based on the user's initial prompt
    
    Args:
        project_id: Project identifier
        name: Project name
    """
    
    # Create data directory structure
    data_dir = os.path.join(settings.projects_root, project_id, "data")
    metadata_dir = os.path.join(data_dir, "metadata")
    ensure_dir(metadata_dir)
    
    metadata_data = {
        "name": name,
        "description": "Project created with AI assistance"
    }
    
    metadata_path = os.path.join(metadata_dir, f"{project_id}.json")
    
    try:
        with open(metadata_path, 'w', encoding='utf-8') as f:
            json.dump(metadata_data, f, indent=2, ensure_ascii=False)
        from claudable_helper.core.terminal_ui import ui
        ui.success(f"Created initial metadata at {metadata_path}", "Project")
    except Exception as e:
        ui.error(f"Failed to create metadata: {e}", "Project")
        raise


async def parse_and_update_project_metadata(project_id: str, db_session) -> dict:
    """
    Parse metadata file and update project information in database
    
    Args:
        project_id: Project identifier
        db_session: Database session
    
    Returns:
        dict: Parsed project information
    """
    
    metadata_path = os.path.join(settings.projects_root, project_id, "data", "metadata", f"{project_id}.json")
    
    if not os.path.exists(metadata_path):
        raise Exception(f"Metadata file not found at {metadata_path}")
    
    try:
        with open(metadata_path, 'r', encoding='utf-8') as f:
            metadata = json.load(f)
        
        # Update project in database
        from claudable_helper.models.projects import Project as ProjectModel
        project = db_session.query(ProjectModel).filter(ProjectModel.id == project_id).first()
        
        if project:
            # Update project fields from metadata
            if metadata.get('name') and metadata['name'] != project.name:
                project.name = metadata['name']
            
            # Store additional info in settings (only description since other fields are pre-configured)
            project.settings = {
                "description": metadata.get('description', ''),
                "features": [],  # Pre-configured
                "tech_stack": ["Next.js", "React", "TypeScript"],  # Pre-configured
                "version": "1.0.0",  # Pre-configured
                "ai_generated": True
            }
            
            db_session.commit()
            ui.success(f"Updated project {project_id} with metadata", "Project")
        
        return metadata
        
    except Exception as e:
        ui.error(f"Failed to parse metadata for project {project_id}: {e}", "Project")
        raise


def get_metadata_path(project_id: str) -> str:
    """Get the metadata file path for a project"""
    return os.path.join(settings.projects_root, project_id, "data", "metadata", f"{project_id}.json")


def setup_claude_config(project_path: str):
    """
    Setup Claude Code configuration for the project
    
    Args:
        project_path: Path to the project repository directory
    """
    try:
        from claudable_helper.core.terminal_ui import ui
        
        # Create .claude directory structure
        claude_dir = os.path.join(project_path, ".claude")
        claude_hooks_dir = os.path.join(claude_dir, "hooks")
        ensure_dir(claude_dir)
        ensure_dir(claude_hooks_dir)
        
        # Get paths to source files in project root
        # Current file: apps/api/app/services/project/initializer.py
        # Go up to project root: ../../../../..
        current_file_dir = os.path.dirname(os.path.abspath(__file__))
        project_root = os.path.join(current_file_dir, "..", "..", "..", "..", "..")
        project_root = os.path.abspath(project_root)
        scripts_dir = os.path.join(project_root, "scripts")
        settings_src = os.path.join(scripts_dir, "settings.json")
        type_check_src = os.path.join(scripts_dir, "type_check.sh")
        
        # Copy settings.json
        settings_dst = os.path.join(claude_dir, "settings.json")
        if os.path.exists(settings_src):
            shutil.copy2(settings_src, settings_dst)
            ui.success(f"Copied settings.json to {settings_dst}", "Claude Config")
        else:
            ui.warning(f"Source file not found: {settings_src}", "Claude Config")
        
        # Copy type_check.sh
        type_check_dst = os.path.join(claude_hooks_dir, "type_check.sh")
        if os.path.exists(type_check_src):
            shutil.copy2(type_check_src, type_check_dst)
            # Make the script executable
            os.chmod(type_check_dst, 0o755)
            ui.success(f"Copied type_check.sh to {type_check_dst}", "Claude Config")
        else:
            ui.warning(f"Source file not found: {type_check_src}", "Claude Config")
        
        ui.success("Claude Code configuration setup complete", "Claude Config")
        
    except Exception as e:
        ui.error(f"Failed to setup Claude configuration: {e}", "Claude Config")
        # Don't fail the whole project creation for this
        pass



"""
Token storage service for local development
"""
import uuid
from datetime import datetime
from typing import Optional
from sqlalchemy.orm import Session
from ..models.tokens import ServiceToken

def save_service_token(
    db: Session, 
    provider: str, 
    token: str, 
    name: str
) -> ServiceToken:
    """Save a service token to database"""
    # Delete existing token for this provider (enforce one token per provider)
    existing = db.query(ServiceToken).filter_by(provider=provider).first()
    if existing:
        db.delete(existing)
    
    # Create new token (plain text for local development)
    service_token = ServiceToken(
        id=str(uuid.uuid4()),
        provider=provider,
        name=name,
        token=token,
        created_at=datetime.utcnow()
    )
    
    db.add(service_token)
    db.commit()
    db.refresh(service_token)
    
    return service_token

def get_service_token(db: Session, provider: str) -> Optional[ServiceToken]:
    """Get service token by provider"""
    return db.query(ServiceToken).filter_by(provider=provider).first()

def get_token(db: Session, provider: str) -> Optional[str]:
    """Get plain text token by provider"""
    service_token = get_service_token(db, provider)
    if service_token:
        return service_token.token
    return None

def delete_service_token(db: Session, token_id: str) -> bool:
    """Delete a service token"""
    token = db.query(ServiceToken).filter_by(id=token_id).first()
    if token:
        db.delete(token)
        db.commit()
        return True
    return False

def update_last_used(db: Session, provider: str):
    """Update last used timestamp for a token"""
    db.query(ServiceToken).filter_by(provider=provider).update({
        "last_used": datetime.utcnow()
    })
    db.commit()

# Legacy function for backward compatibility
def get_decrypted_token(db: Session, provider: str) -> Optional[str]:
    """Legacy function - use get_token instead"""
    return get_token(db, provider)

class TokenService:
    """Token service class for compatibility"""
    
    def save_service_token(self, db: Session, provider: str, token: str, name: str) -> ServiceToken:
        return save_service_token(db, provider, token, name)
    
    def get_service_token(self, db: Session, provider: str) -> Optional[ServiceToken]:
        return get_service_token(db, provider)
    
    def get_token(self, db: Session, provider: str) -> Optional[str]:
        return get_token(db, provider)
    
    def get_decrypted_token(self, db: Session, provider: str) -> Optional[str]:
        """Legacy method - use get_token instead"""
        return get_token(db, provider)
    
    def delete_service_token(self, db: Session, token_id: str) -> bool:
        return delete_service_token(db, token_id)
    
    def update_last_used(self, db: Session, provider: str):
        return update_last_used(db, provider)
    
    async def get_token_async(self, provider: str, db: Session = None) -> Optional[dict]:
        """Get token for a provider - async version for compatibility"""
        if db is None:
            return None
        
        token = self.get_token(db, provider)
        if token:
            return {"token": token, "provider": provider}
        return None


"""Mock configuration module for claudable_helper.

This module provides a mock implementation of the configuration interface
that was originally imported from app.core.config.
"""
import os
from typing import Any, Optional


class MockSettings:
    """Mock settings object for configuration."""
    
    def __init__(self):
        self._values = {}
    
    def __getattr__(self, name: str) -> Any:
        """Get setting value with fallback to environment variables."""
        # First check if we have it stored
        if name in self._values:
            return self._values[name]
        
        # Try environment variable (uppercase)
        env_name = name.upper()
        env_value = os.environ.get(env_name)
        if env_value is not None:
            return env_value
        
        # Common defaults for known settings
        defaults = {
            "debug": False,
            "environment": "development",
            "base_url": "http://localhost:8000",
            "project_root": os.getcwd(),
        }
        
        return defaults.get(name, None)
    
    def __setattr__(self, name: str, value: Any) -> None:
        """Set setting value."""
        if name.startswith('_'):
            super().__setattr__(name, value)
        else:
            if not hasattr(self, '_values'):
                super().__setattr__('_values', {})
            self._values[name] = value
    
    def get(self, name: str, default: Any = None) -> Any:
        """Get setting with default value."""
        try:
            return getattr(self, name)
        except AttributeError:
            return default


# Create singleton instance
settings = MockSettings()


"""
Environment Variables Manager

Handles synchronization between database and .env files in Next.js projects.
"""

import os
import re
from pathlib import Path
from typing import Dict, List, Optional, Tuple
from sqlalchemy.orm import Session
from ..models.env_vars import EnvVar
from ..core.crypto import secret_box
from ..core.config import settings


def get_project_env_path(project_id: str) -> Path:
    """Get the path to project's .env file"""
    return Path(settings.projects_root) / project_id / "repo" / ".env"


def parse_env_file(env_path: Path) -> Dict[str, str]:
    """Parse .env file and return key-value pairs"""
    env_vars = {}
    
    if not env_path.exists():
        return env_vars
    
    try:
        with open(env_path, 'r', encoding='utf-8') as f:
            for line_num, line in enumerate(f, 1):
                line = line.strip()
                
                # Skip empty lines and comments
                if not line or line.startswith('#'):
                    continue
                
                # Match KEY=VALUE pattern
                match = re.match(r'^([A-Za-z_][A-Za-z0-9_]*)\s*=\s*(.*)$', line)
                if match:
                    key, value = match.groups()
                    
                    # Handle quoted values
                    if value.startswith('"') and value.endswith('"'):
                        value = value[1:-1]
                    elif value.startswith("'") and value.endswith("'"):
                        value = value[1:-1]
                    
                    env_vars[key] = value
                
    except Exception as e:
        print(f"Error parsing .env file {env_path}: {e}")
    
    return env_vars


def write_env_file(env_path: Path, env_vars: Dict[str, str]) -> None:
    """Write environment variables to .env file"""
    try:
        # Ensure directory exists
        env_path.parent.mkdir(parents=True, exist_ok=True)
        
        # Write .env file
        with open(env_path, 'w', encoding='utf-8') as f:
            f.write("# Environment Variables\n")
            f.write("# This file is automatically synchronized with Project Settings\n\n")
            
            # Sort keys for consistent output
            for key in sorted(env_vars.keys()):
                value = env_vars[key]
                
                # Quote values that contain spaces or special characters
                if ' ' in value or any(c in value for c in ['#', '$', '`', '"', "'"]):
                    value = f'"{value}"'
                
                f.write(f"{key}={value}\n")
                
        from ..core.terminal_ui import ui
        ui.success(f"Updated .env file: {env_path}", "EnvManager")
        
    except Exception as e:
        ui.error(f"Error writing .env file {env_path}: {e}", "EnvManager")
        raise


def load_env_vars_from_db(db: Session, project_id: str) -> Dict[str, str]:
    """Load environment variables from database for a project"""
    env_vars = {}
    
    try:
        db_env_vars = db.query(EnvVar).filter(
            EnvVar.project_id == project_id
        ).all()
        
        for env_var in db_env_vars:
            try:
                # Decrypt the value
                decrypted_value = secret_box.decrypt(env_var.value_encrypted)
                env_vars[env_var.key] = decrypted_value
            except Exception as e:
                print(f"⚠️  Failed to decrypt env var {env_var.key}: {e}")
                
    except Exception as e:
        from ..core.terminal_ui import ui
        ui.error(f"Error loading env vars from DB for project {project_id}: {e}", "EnvManager")
    
    return env_vars


def sync_env_file_to_db(db: Session, project_id: str) -> int:
    """
    Sync .env file contents to database (file -> DB)
    Returns number of variables synced
    """
    env_path = get_project_env_path(project_id)
    file_env_vars = parse_env_file(env_path)
    
    synced_count = 0
    
    try:
        # Get existing env vars from DB
        existing_vars = {
            env_var.key: env_var 
            for env_var in db.query(EnvVar).filter(EnvVar.project_id == project_id).all()
        }
        
        # Update or create env vars from file
        for key, value in file_env_vars.items():
            if key in existing_vars:
                # Update existing
                existing_var = existing_vars[key]
                try:
                    # Only update if value changed
                    current_value = secret_box.decrypt(existing_var.value_encrypted)
                    if current_value != value:
                        existing_var.value_encrypted = secret_box.encrypt(value)
                        synced_count += 1
                except Exception as e:
                    print(f"⚠️  Failed to decrypt existing value for {key}: {e}")
                    existing_var.value_encrypted = secret_box.encrypt(value)
                    synced_count += 1
            else:
                # Create new
                import uuid
                new_env_var = EnvVar(
                    id=str(uuid.uuid4()),
                    project_id=project_id,
                    key=key,
                    value_encrypted=secret_box.encrypt(value),
                    scope="runtime",
                    var_type="string",
                    is_secret=True
                )
                db.add(new_env_var)
                synced_count += 1
        
        # Remove env vars from DB that are not in file
        file_keys = set(file_env_vars.keys())
        for key, existing_var in existing_vars.items():
            if key not in file_keys:
                db.delete(existing_var)
                synced_count += 1
        
        db.commit()
        from ..core.terminal_ui import ui
        ui.success(f"Synced {synced_count} env vars from file to DB", "EnvManager")
        
    except Exception as e:
        ui.error(f"Error syncing env file to DB: {e}", "EnvManager")
        db.rollback()
        raise
    
    return synced_count


def sync_db_to_env_file(db: Session, project_id: str) -> int:
    """
    Sync database contents to .env file (DB -> file)
    Returns number of variables synced
    """
    try:
        # Load from database
        env_vars = load_env_vars_from_db(db, project_id)
        
        # Write to file
        env_path = get_project_env_path(project_id)
        write_env_file(env_path, env_vars)
        
        print(f"✅ Synced {len(env_vars)} env vars from DB to file")
        return len(env_vars)
        
    except Exception as e:
        print(f"❌ Error syncing DB to env file: {e}")
        raise


def get_env_var_conflicts(db: Session, project_id: str) -> List[Dict]:
    """
    Check for conflicts between DB and .env file
    Returns list of conflicts with details
    """
    conflicts = []
    
    try:
        env_path = get_project_env_path(project_id)
        file_env_vars = parse_env_file(env_path)
        db_env_vars = load_env_vars_from_db(db, project_id)
        
        # Check for differences
        all_keys = set(file_env_vars.keys()) | set(db_env_vars.keys())
        
        for key in all_keys:
            file_value = file_env_vars.get(key)
            db_value = db_env_vars.get(key)
            
            if file_value != db_value:
                conflicts.append({
                    "key": key,
                    "file_value": file_value,
                    "db_value": db_value,
                    "conflict_type": (
                        "file_only" if file_value and not db_value else
                        "db_only" if db_value and not file_value else
                        "value_mismatch"
                    )
                })
    
    except Exception as e:
        print(f"❌ Error checking env var conflicts: {e}")
    
    return conflicts


def create_env_var(db: Session, project_id: str, key: str, value: str, 
                   scope: str = "runtime", var_type: str = "string", 
                   is_secret: bool = True, description: Optional[str] = None) -> EnvVar:
    """Create a new environment variable and sync to file"""
    import uuid
    
    # Create in database
    env_var = EnvVar(
        id=str(uuid.uuid4()),
        project_id=project_id,
        key=key,
        value_encrypted=secret_box.encrypt(value),
        scope=scope,
        var_type=var_type,
        is_secret=is_secret,
        description=description
    )
    
    db.add(env_var)
    db.commit()
    
    # Sync to file
    sync_db_to_env_file(db, project_id)
    
    return env_var


def update_env_var(db: Session, project_id: str, key: str, value: str) -> bool:
    """Update an environment variable and sync to file"""
    env_var = db.query(EnvVar).filter(
        EnvVar.project_id == project_id,
        EnvVar.key == key
    ).first()
    
    if not env_var:
        return False
    
    # Update in database
    env_var.value_encrypted = secret_box.encrypt(value)
    db.commit()
    
    # Sync to file
    sync_db_to_env_file(db, project_id)
    
    return True


def delete_env_var(db: Session, project_id: str, key: str) -> bool:
    """Delete an environment variable and sync to file"""
    env_var = db.query(EnvVar).filter(
        EnvVar.project_id == project_id,
        EnvVar.key == key
    ).first()
    
    if not env_var:
        return False
    
    # Delete from database
    db.delete(env_var)
    db.commit()
    
    # Sync to file
    sync_db_to_env_file(db, project_id)
    
    return True


"""Mock terminal UI implementation for claudable_helper.

This module provides a mock implementation of the terminal UI interface
that was originally imported from app.core.terminal_ui.
"""
import sys
from typing import Any, Optional


class MockTerminalUI:
    """Mock terminal UI for logging and user interaction."""
    
    def debug(self, message: str, category: str = "DEBUG") -> None:
        """Log debug message."""
        if self._should_show_debug():
            print(f"[{category}] {message}", file=sys.stderr)
    
    def info(self, message: str, category: str = "INFO") -> None:
        """Log info message."""
        print(f"[{category}] {message}")
    
    def warning(self, message: str, category: str = "WARNING") -> None:
        """Log warning message."""
        print(f"[{category}] {message}", file=sys.stderr)
    
    def error(self, message: str, category: str = "ERROR") -> None:
        """Log error message."""
        print(f"[{category}] {message}", file=sys.stderr)
    
    def success(self, message: str, category: str = "SUCCESS") -> None:
        """Log success message."""
        print(f"[{category}] {message}")
    
    def print(self, message: str = "", **kwargs) -> None:
        """Print message to stdout."""
        print(message, **kwargs)
    
    def input(self, prompt: str = "") -> str:
        """Get user input."""
        return input(prompt)
    
    def confirm(self, message: str, default: bool = True) -> bool:
        """Get user confirmation."""
        default_text = "Y/n" if default else "y/N"
        response = input(f"{message} [{default_text}]: ").lower().strip()
        
        if not response:
            return default
        return response.startswith('y')
    
    def select(self, message: str, choices: list, default: Optional[int] = None) -> int:
        """Select from a list of choices."""
        print(message)
        for i, choice in enumerate(choices):
            marker = " (default)" if default == i else ""
            print(f"{i + 1}. {choice}{marker}")
        
        while True:
            try:
                response = input("Select option: ").strip()
                if not response and default is not None:
                    return default
                
                choice_idx = int(response) - 1
                if 0 <= choice_idx < len(choices):
                    return choice_idx
                else:
                    print(f"Please enter a number between 1 and {len(choices)}")
            except ValueError:
                print("Please enter a valid number")
    
    def _should_show_debug(self) -> bool:
        """Check if debug messages should be shown."""
        import os
        return os.environ.get("DEBUG", "").lower() in ("1", "true", "yes")


# Create singleton instance
ui = MockTerminalUI()


import subprocess
from typing import List, Optional
import os


def _run(cmd: list[str], cwd: str) -> str:
    res = subprocess.run(cmd, cwd=cwd, check=True, capture_output=True, text=True)
    return res.stdout.strip()


def list_commits(repo_path: str, limit: int = 50) -> list[dict]:
    fmt = "%H%x01%P%x01%an%x01%ad%x01%s"
    out = _run(["git", "log", f"-n{limit}", f"--pretty=format:{fmt}", "--date=iso"], cwd=repo_path)
    commits: list[dict] = []
    if not out:
        return commits
    for line in out.splitlines():
        sha, parents, author, date, subject = line.split("\x01")
        commits.append({
            "commit_sha": sha,
            "parent_sha": parents.split()[0] if parents else None,
            "author": author,
            "date": date,
            "message": subject,
        })
    return commits


def show_diff(repo_path: str, commit_sha: str) -> str:
    return _run(["git", "show", "--format=", commit_sha], cwd=repo_path)


def current_head(repo_path: str) -> str:
    return _run(["git", "rev-parse", "HEAD"], cwd=repo_path)


# Legacy function for backward compatibility
def commit_all_legacy(repo_path: str, message: str) -> str:
    _run(["git", "add", "-A"], cwd=repo_path)
    _run(["git", "commit", "-m", message], cwd=repo_path)
    return current_head(repo_path)


def hard_reset(repo_path: str, commit_sha: str) -> None:
    _run(["git", "reset", "--hard", commit_sha], cwd=repo_path)


def add_remote(repo_path: str, remote_name: str, remote_url: str) -> None:
    """Add a remote repository"""
    try:
        # Check if remote already exists
        existing_url = _run(["git", "remote", "get-url", remote_name], cwd=repo_path)
        
        # Compare URLs without authentication credentials for proper comparison
        def normalize_url(url):
            # Remove credentials from URL for comparison
            import re
            return re.sub(r'https://[^@]+@github.com/', 'https://github.com/', url)
        
        if normalize_url(existing_url) != normalize_url(remote_url):
            # Different repository - remove existing remote and add new one
            _run(["git", "remote", "remove", remote_name], cwd=repo_path)
            _run(["git", "remote", "add", remote_name, remote_url], cwd=repo_path)
            
            # Unset any existing upstream to avoid conflicts
            try:
                _run(["git", "branch", "--unset-upstream"], cwd=repo_path)
            except subprocess.CalledProcessError:
                pass  # No upstream set, that's fine
        else:
            # Same repository but potentially different credentials - update URL
            _run(["git", "remote", "set-url", remote_name, remote_url], cwd=repo_path)
    except subprocess.CalledProcessError:
        # Remote doesn't exist, add it
        _run(["git", "remote", "add", remote_name, remote_url], cwd=repo_path)


def push_to_remote(repo_path: str, remote_name: str = "origin", branch: str = "main") -> dict:
    """Push to remote repository"""
    try:
        # First try normal push with upstream
        try:
            result = _run(["git", "push", "-u", remote_name, branch], cwd=repo_path)
        except subprocess.CalledProcessError:
            # If push fails (e.g., different histories), try force push
            # This is safe for initial connection to a new empty repo
            result = _run(["git", "push", "-u", "--force", remote_name, branch], cwd=repo_path)
            
        return {
            "success": True,
            "output": result,
            "remote": remote_name,
            "branch": branch
        }
    except subprocess.CalledProcessError as e:
        return {
            "success": False,
            "error": e.stderr if e.stderr else str(e),
            "remote": remote_name,
            "branch": branch
        }


def get_remote_url(repo_path: str, remote_name: str = "origin") -> str:
    """Get remote URL"""
    try:
        return _run(["git", "remote", "get-url", remote_name], cwd=repo_path)
    except subprocess.CalledProcessError:
        return ""


def get_current_branch(repo_path: str) -> str:
    """Get current branch name"""
    try:
        return _run(["git", "branch", "--show-current"], cwd=repo_path)
    except subprocess.CalledProcessError:
        return "main"  # fallback to main


def set_git_config(repo_path: str, name: str, email: str) -> None:
    """Set git config for the repository"""
    # Set local repository config (not global)
    _run(["git", "config", "--local", "user.name", name], cwd=repo_path)
    _run(["git", "config", "--local", "user.email", email], cwd=repo_path)


def initialize_main_branch(repo_path: str) -> None:
    """Initialize main branch if not exists"""
    try:
        # Check if we have any commits
        _run(["git", "rev-parse", "HEAD"], cwd=repo_path)
    except subprocess.CalledProcessError:
        # No commits yet, create initial commit
        _run(["git", "add", "."], cwd=repo_path)
        try:
            _run(["git", "commit", "-m", "Initial commit"], cwd=repo_path)
        except subprocess.CalledProcessError:
            # Nothing to commit, create empty commit
            _run(["git", "commit", "--allow-empty", "-m", "Initial commit"], cwd=repo_path)
    
    # Ensure we're on main branch
    try:
        current_branch = get_current_branch(repo_path)
        if current_branch != "main":
            try:
                _run(["git", "branch", "-M", "main"], cwd=repo_path)
            except subprocess.CalledProcessError:
                # Branch rename failed, checkout main
                try:
                    _run(["git", "checkout", "-b", "main"], cwd=repo_path)
                except subprocess.CalledProcessError:
                    pass  # Already on main or other issue
    except subprocess.CalledProcessError:
        pass


def commit_all(repo_path: str, message: str) -> dict:
    """Stage all changes and commit, return commit info"""
    try:
        _run(["git", "add", "-A"], cwd=repo_path)
        _run(["git", "commit", "-m", message], cwd=repo_path)
        commit_sha = current_head(repo_path)
        return {
            "success": True,
            "commit_hash": commit_sha,
            "message": message
        }
    except subprocess.CalledProcessError as e:
        return {
            "success": False,
            "error": str(e),
            "message": message
        }



"""Mock crypto module for claudable_helper.

This module provides a mock implementation of the crypto interface
that was originally imported from app.core.crypto.
"""
import base64
import os
from typing import Union


class MockSecretBox:
    """Mock secret box for encryption/decryption."""
    
    def __init__(self):
        # In a real implementation, this would use proper encryption
        # For this mock, we'll just use base64 encoding as a placeholder
        pass
    
    def encrypt(self, plaintext: Union[str, bytes]) -> bytes:
        """Mock encrypt function - just base64 encode for demo."""
        if isinstance(plaintext, str):
            plaintext = plaintext.encode('utf-8')
        
        # Add a simple prefix to distinguish encrypted data
        prefixed = b"MOCK_ENCRYPTED:" + plaintext
        return base64.b64encode(prefixed)
    
    def decrypt(self, ciphertext: bytes) -> bytes:
        """Mock decrypt function - just base64 decode for demo."""
        try:
            decoded = base64.b64decode(ciphertext)
            if decoded.startswith(b"MOCK_ENCRYPTED:"):
                return decoded[15:]  # Remove prefix
            return decoded
        except Exception:
            # If decryption fails, return empty bytes
            return b""
    
    def decrypt_str(self, ciphertext: bytes) -> str:
        """Decrypt and return as string."""
        decrypted = self.decrypt(ciphertext)
        try:
            return decrypted.decode('utf-8')
        except UnicodeDecodeError:
            return ""


# Create singleton instance
secret_box = MockSecretBox()


"""Mock project service models for claudable_helper.

This module provides mock implementations of the project service models
that were originally imported from app.models.project_services.
"""
import uuid
from datetime import datetime
from enum import Enum
from typing import Any, Dict, List, Optional


class ServiceType(str, Enum):
    """Service type enumeration."""
    DATABASE = "database"
    CACHE = "cache"
    MESSAGE_QUEUE = "message_queue"
    STORAGE = "storage"
    API = "api"
    CDN = "cdn"
    MONITORING = "monitoring"
    DEPLOYMENT = "deployment"
    OTHER = "other"


class ConnectionStatus(str, Enum):
    """Connection status enumeration."""
    CONNECTED = "connected"
    DISCONNECTED = "disconnected"
    ERROR = "error"
    UNKNOWN = "unknown"


class ProjectServiceConnection:
    """Mock project service connection model."""
    
    def __init__(
        self,
        project_id: str,
        service_name: str,
        service_type: ServiceType,
        connection_id: Optional[str] = None,
        connection_string: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
        credentials: Optional[Dict[str, str]] = None,
        metadata: Optional[Dict[str, Any]] = None,
        created_at: Optional[datetime] = None,
        updated_at: Optional[datetime] = None,
    ):
        self.id = connection_id or str(uuid.uuid4())
        self.project_id = project_id
        self.service_name = service_name
        self.service_type = service_type
        self.connection_string = connection_string
        self.config = config or {}
        self.credentials = credentials or {}
        self.metadata = metadata or {}
        self.created_at = created_at or datetime.utcnow()
        self.updated_at = updated_at or datetime.utcnow()
        
        # Runtime status
        self._status = ConnectionStatus.UNKNOWN
        self._last_check: Optional[datetime] = None
    
    @property
    def status(self) -> ConnectionStatus:
        """Get current connection status."""
        return self._status
    
    @property
    def last_check(self) -> Optional[datetime]:
        """Get timestamp of last status check."""
        return self._last_check
    
    @property
    def is_connected(self) -> bool:
        """Check if service is currently connected."""
        return self._status == ConnectionStatus.CONNECTED
    
    @property
    def masked_connection_string(self) -> Optional[str]:
        """Get masked version of connection string for display."""
        if not self.connection_string:
            return None
        
        # Simple masking - replace passwords and sensitive info
        masked = self.connection_string
        
        # Mask password in connection strings
        if "://" in masked:
            try:
                scheme, rest = masked.split("://", 1)
                if "@" in rest:
                    auth, host_part = rest.split("@", 1)
                    if ":" in auth:
                        user, _ = auth.split(":", 1)
                        masked = f"{scheme}://{user}:***@{host_part}"
            except ValueError:
                pass  # Keep original if parsing fails
        
        return masked
    
    async def test_connection(self) -> bool:
        """Test the service connection (mock implementation)."""
        # In a real implementation, this would actually test the connection
        # For mock, we'll simulate success most of the time
        import random
        
        success = random.random() > 0.1  # 90% success rate
        self._status = ConnectionStatus.CONNECTED if success else ConnectionStatus.ERROR
        self._last_check = datetime.utcnow()
        self.updated_at = datetime.utcnow()
        
        return success
    
    def update_config(self, config: Dict[str, Any]) -> None:
        """Update service configuration."""
        self.config.update(config)
        self.updated_at = datetime.utcnow()
    
    def update_credentials(self, credentials: Dict[str, str]) -> None:
        """Update service credentials."""
        self.credentials.update(credentials)
        self.updated_at = datetime.utcnow()
    
    def get_config_value(self, key: str, default: Any = None) -> Any:
        """Get configuration value."""
        return self.config.get(key, default)
    
    def set_config_value(self, key: str, value: Any) -> None:
        """Set configuration value."""
        self.config[key] = value
        self.updated_at = datetime.utcnow()
    
    def to_dict(self, include_credentials: bool = False, mask_sensitive: bool = True) -> Dict[str, Any]:
        """Convert service connection to dictionary.
        
        Args:
            include_credentials: Whether to include credentials
            mask_sensitive: Whether to mask sensitive information
        """
        data = {
            "id": self.id,
            "project_id": self.project_id,
            "service_name": self.service_name,
            "service_type": self.service_type.value,
            "config": self.config,
            "metadata": self.metadata,
            "status": self.status.value,
            "last_check": self.last_check.isoformat() if self.last_check else None,
            "created_at": self.created_at.isoformat() if self.created_at else None,
            "updated_at": self.updated_at.isoformat() if self.updated_at else None,
        }
        
        if self.connection_string:
            if mask_sensitive:
                data["connection_string"] = self.masked_connection_string
            else:
                data["connection_string"] = self.connection_string
        
        if include_credentials:
            data["credentials"] = self.credentials
        
        return data
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ProjectServiceConnection":
        """Create service connection from dictionary."""
        return cls(
            connection_id=data.get("id"),
            project_id=data.get("project_id", ""),
            service_name=data.get("service_name", ""),
            service_type=ServiceType(data.get("service_type", "other")),
            connection_string=data.get("connection_string"),
            config=data.get("config", {}),
            credentials=data.get("credentials", {}),
            metadata=data.get("metadata", {}),
            created_at=datetime.fromisoformat(data["created_at"]) if data.get("created_at") else None,
            updated_at=datetime.fromisoformat(data["updated_at"]) if data.get("updated_at") else None,
        )
    
    def __str__(self) -> str:
        """String representation of the service connection."""
        return f"ProjectServiceConnection(service={self.service_name}, type={self.service_type.value})"
    
    def __repr__(self) -> str:
        """Detailed string representation."""
        return (
            f"ProjectServiceConnection(id={self.id}, project_id={self.project_id}, "
            f"service={self.service_name}, type={self.service_type.value}, status={self.status.value})"
        )